@@ -17,15 +17,19 @@ from autograd.numpy import (  # NOQA
     argmax,
     argmin,
     array,
+    broadcast_arrays,
     ceil,
     clip,
     concatenate,
     cos,
     cosh,
+    cross,
+    cumprod,
     cumsum,
     diagonal,
     divide,
     dot,
+    dtype,
     einsum,
     empty,
     empty_like,
@@ -43,6 +47,7 @@ from autograd.numpy import (  # NOQA
     int32,
     int64,
     isclose,
+    isnan,
     less,
     less_equal,
     linspace,
@@ -85,12 +90,33 @@ from autograd.numpy import (  # NOQA
     zeros,
     zeros_like
 )
+from autograd.scipy.special import polygamma # NOQA
 from scipy.sparse import coo_matrix
 
 from . import linalg  # NOQA
 from . import random  # NOQA
 from .common import to_ndarray  # NOQA
 
+DTYPES = {
+    dtype('int32'): 0,
+    dtype('int64'): 1,
+    dtype('float32'): 2,
+    dtype('float64'): 3}
+
+
+def to_numpy(x):
+    return x
+
+
+def convert_to_wider_dtype(tensor_list):
+    dtype_list = [DTYPES[x.dtype] for x in tensor_list]
+    wider_dtype_index = max(dtype_list)
+
+    wider_dtype = list(DTYPES.keys())[wider_dtype_index]
+
+    tensor_list = [cast(x, dtype=wider_dtype) for x in tensor_list]
+    return tensor_list
+
 
 def flatten(x):
     return x.flatten()
@@ -118,6 +144,24 @@ def get_mask_i_float(i, n):
     return mask_i_float
 
 
+def _is_boolean(x):
+    if isinstance(x, bool):
+        return True
+    if isinstance(x, (tuple, list)):
+        return _is_boolean(x[0])
+    if isinstance(x, np.ndarray):
+        return x.dtype == bool
+    return False
+
+
+def _is_iterable(x):
+    if isinstance(x, (list, tuple)):
+        return True
+    if isinstance(x, np.ndarray):
+        return ndim(x) > 0
+    return False
+
+
 def assignment(x, values, indices, axis=0):
     """Assign values at given indices of an array.
 
@@ -146,19 +190,26 @@ def assignment(x, values, indices, axis=0):
     If a list is given, it must have the same length as indices.
     """
     x_new = copy(x)
-    if not isinstance(indices, list):
-        indices = [indices]
-    if not isinstance(values, list):
-        values = [values] * len(indices)
-    for nb_index, index in enumerate(indices):
-        if not isinstance(index, tuple):
-            index = (index,)
-        if len(index) < len(shape(x)):
-            for n_axis in range(shape(x)[axis]):
-                extended_index = index[:axis] + (n_axis,) + index[axis:]
-                x_new[extended_index] = values[nb_index]
-        else:
-            x_new[index] = values[nb_index]
+
+    use_vectorization = hasattr(indices, '__len__') and len(indices) < ndim(x)
+    if _is_boolean(indices):
+        x_new[indices] = values
+        return x_new
+    zip_indices = _is_iterable(indices) and _is_iterable(indices[0])
+    len_indices = len(indices) if _is_iterable(indices) else 1
+    if zip_indices:
+        indices = tuple(zip(*indices))
+    if not use_vectorization:
+        if not zip_indices:
+            len_indices = len(indices) if _is_iterable(indices) else 1
+        len_values = len(values) if _is_iterable(values) else 1
+        if len_values > 1 and len_values != len_indices:
+            raise ValueError('Either one value or as many values as indices')
+        x_new[indices] = values
+    else:
+        indices = tuple(
+            list(indices[:axis]) + [slice(None)] + list(indices[axis:]))
+        x_new[indices] = values
     return x_new
 
 
@@ -190,19 +241,24 @@ def assignment_by_sum(x, values, indices, axis=0):
     If a list is given, it must have the same length as indices.
     """
     x_new = copy(x)
-    if not isinstance(indices, list):
-        indices = [indices]
-    if not isinstance(values, list):
-        values = [values] * len(indices)
-    for nb_index, index in enumerate(indices):
-        if not isinstance(index, tuple):
-            index = (index,)
-        if len(index) < len(shape(x)):
-            for n_axis in range(shape(x)[axis]):
-                extended_index = index[:axis] + (n_axis,) + index[axis:]
-                x_new[extended_index] += values[nb_index]
-        else:
-            x_new[index] += values[nb_index]
+
+    use_vectorization = hasattr(indices, '__len__') and len(indices) < ndim(x)
+    if _is_boolean(indices):
+        x_new[indices] += values
+        return x_new
+    zip_indices = _is_iterable(indices) and _is_iterable(indices[0])
+    if zip_indices:
+        indices = tuple(zip(*indices))
+    if not use_vectorization:
+        len_indices = len(indices) if _is_iterable(indices) else 1
+        len_values = len(values) if _is_iterable(values) else 1
+        if len_values > 1 and len_values != len_indices:
+            raise ValueError('Either one value or as many values as indices')
+        x_new[indices] += values
+    else:
+        indices = tuple(
+            list(indices[:axis]) + [slice(None)] + list(indices[axis:]))
+        x_new[indices] += values
     return x_new
 
 
@@ -296,3 +352,12 @@ def array_from_sparse(indices, data, target_shape):
     """
     return array(
         coo_matrix((data, list(zip(*indices))), target_shape).todense())
+
+
+def erf(x):
+    cst_erf = 8.0 / (3.0 * np.pi) * (np.pi - 3.0) / (4.0 - np.pi)
+    return \
+        np.sign(x) * \
+        np.sqrt(1 - np.exp(-x * x *
+                           (4 / np.pi + cst_erf * x * x) /
+                           (1 + cst_erf * x * x)))
