@@ -7,5 +7,7 @@ def to_ndarray(x, to_ndim, axis=0):
         x = np.expand_dims(x, axis=axis)
 
     if x.ndim != 0:
-        assert x.ndim >= to_ndim
+        if x.ndim < to_ndim:
+            raise ValueError(
+                'The ndim was not adapted properly.')
     return x
