@@ -50,11 +50,17 @@ def det(*args, **kwargs):
     return torch.from_numpy(np.array(np.linalg.det(*args, **kwargs)))
 
 
-def norm(x, ord=2, axis=None, keepdims=False):
+def norm(x, ord=2, axis=None):
     if axis is None:
         return torch.norm(x, p=ord)
     return torch.norm(x, p=ord, dim=axis)
 
 
 def qr(*args, **kwargs):
-    return torch.from_numpy(np.linalg.qr(*args, **kwargs))
+    matrix_q, matrix_r = np.vectorize(
+        np.linalg.qr,
+        signature='(n,m)->(n,k),(k,m)',
+        excluded=['mode'])(*args, **kwargs)
+    tensor_q = torch.from_numpy(matrix_q)
+    tensor_r = torch.from_numpy(matrix_r)
+    return tensor_q, tensor_r
