@@ -28,6 +28,6 @@ def normal(loc=0.0, scale=1.0, size=(1,)):
 def uniform(low=0.0, high=1.0, size=(1,)):
     if not hasattr(size, '__iter__'):
         size = (size,)
-    if low <= high:
+    if low >= high:
         raise ValueError('Upper bound must be higher than lower bound')
     return (high - low) * torch.rand(*size) + low
