@@ -3,11 +3,10 @@
 import random
 
 import geomstats.backend as gs
+import geomstats.datasets.utils as data_utils
 
-DEFAULT_GRAPH_MATRIX_PATH = 'examples/data' \
-                            '/graph_random/graph_random.txt'
-DEFAULT_GRAPH_LABELS_PATH = 'examples/data' \
-                            '/graph_random/graph_random_labels.txt'
+GRAPH_MATRIX_PATH = data_utils.GRAPH_RANDOM_PATH
+GRAPH_LABELS_PATH = data_utils.GRAPH_RANDOM_LABELS_PATH
 
 
 class Graph:
@@ -38,14 +37,14 @@ class Graph:
     labels = None
 
     def __init__(self,
-                 graph_matrix_path=DEFAULT_GRAPH_MATRIX_PATH,
-                 labels_path=DEFAULT_GRAPH_LABELS_PATH):
+                 graph_matrix_path=GRAPH_MATRIX_PATH,
+                 labels_path=GRAPH_LABELS_PATH):
         self.edges = {}
         with open(graph_matrix_path, 'r') as edges_file:
             for i, line in enumerate(edges_file):
                 lsp = line.split()
                 self.edges[i] = [k for k, value in
-                                 enumerate(lsp) if (int(value) == 1)]
+                                 enumerate(lsp) if int(value) == 1]
 
         self.n_nodes = len(self.edges)
 
@@ -78,7 +77,7 @@ class Graph:
             array containing random walks.
         """
         paths = gs.empty(
-            (self.n_nodes * n_walks_per_node, walk_length + 1), dtype=int)
+            (self.n_nodes * n_walks_per_node, walk_length + 1), dtype=gs.int32)
         for index in range(len(self.edges)):
             for i in range(n_walks_per_node):
                 paths[index * n_walks_per_node + i] =\
@@ -93,4 +92,4 @@ class Graph:
             count_index = self.edges[count_index][random.randint(
                 0, len(self.edges[count_index]) - 1)]
             path.append(count_index)
-        return gs.array(path).astype(gs.int32)
+        return gs.array(path, dtype=gs.int32)
