@@ -4,7 +4,8 @@ import autograd
 from scipy.optimize import minimize
 
 import geomstats.backend as gs
-import geomstats.error
+import geomstats.errors
+import geomstats.vectorization
 from geomstats.integrator import integrate
 
 
@@ -13,19 +14,25 @@ EPSILON = 1e-3
 
 
 class Connection:
-    """Class for affine connections.
+    r"""Class for affine connections.
 
     Parameters
     ----------
-    dim: int
+    dim : int
         Dimension of the underlying manifold.
+    default_point_type : str, {\'vector\', \'matrix\'}
+        Point type.
+        Optional, default: \'vector\'.
+    default_coords_type : str, {\'intrinsic\', \'extrinsic\', etc}
+        Coordinate type.
+        Optional, default: \'intrinsic\'.
     """
 
     def __init__(
             self, dim, default_point_type='vector',
             default_coords_type='intrinsic'):
-        geomstats.error.check_integer(dim, 'dim')
-        geomstats.error.check_parameter_accepted_values(
+        geomstats.errors.check_integer(dim, 'dim')
+        geomstats.errors.check_parameter_accepted_values(
             default_point_type, 'default_point_type', ['vector', 'matrix'])
 
         self.dim = dim
@@ -37,13 +44,13 @@ class Connection:
 
         Parameters
         ----------
-        base_point : array-like, shape=[n_samples, dim]
+        base_point : array-like, shape=[..., dim]
             Point on the manifold.
 
         Returns
         -------
-        gamma : array-like, shape=[n_samples, dim, dim, dim]
-            Values of the christoffel symbols, with the covariant index on
+        gamma : array-like, shape=[..., dim, dim, dim]
+            Christoffel symbols, with the covariant index on
             the first dimension.
         """
         raise NotImplementedError(
@@ -57,11 +64,11 @@ class Connection:
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[n_samples, dim]
+        tangent_vec_a : array-like, shape=[..., dim]
             Tangent vector at base point.
-        tangent_vec_b : array-like, shape=[n_samples, dim]
+        tangent_vec_b : array-like, shape=[..., dim]
             Tangent vector at base point.
-        base_point : array-like, shape=[n_samples, dim]
+        base_point : array-like, shape=[..., dim]
             Point on the manifold.
         """
         raise NotImplementedError(
@@ -72,20 +79,23 @@ class Connection:
 
         Parameters
         ----------
-        velocity : array-like, shape=[n_samples, dim]
+        velocity : array-like, shape=[..., dim]
             Tangent vector at the position.
-        position : array-like, shape=[n_samples, dim]
+        position : array-like, shape=[..., dim]
             Point on the manifold, the position at which to compute the
             geodesic ODE.
 
         Returns
         -------
-        geodesic_ode : array-like, shape=[n_samples, dim]
+        geodesic_ode : array-like, shape=[..., dim]
             Value of the vector field to be integrated at position.
         """
         gamma = self.christoffels(position)
-        return - gs.einsum('...kij,...i,...j->...k', gamma, velocity,
-                           velocity)
+        equation = gs.einsum(
+            '...kij,...i->...kj', gamma, velocity)
+        equation = - gs.einsum(
+            '...kj,...j->...k', equation, velocity)
+        return equation
 
     def exp(self, tangent_vec, base_point, n_steps=N_STEPS, step='euler',
             point_type=None):
@@ -97,28 +107,31 @@ class Connection:
 
         Parameters
         ----------
-        tangent_vec : array-like, shape=[n_samples, dim]
+        tangent_vec : array-like, shape=[..., dim]
             Tangent vector at the base point.
-        base_point : array-like, shape=[n_samples, dim]
+        base_point : array-like, shape=[..., dim]
             Point on the manifold.
         n_steps : int
-            The number of discrete time steps to take in the integration.
+            Number of discrete time steps to take in the integration.
+            Optional, default: N_STEPS.
         step : str, {'euler', 'rk4'}
             The numerical scheme to use for integration.
+            Optional, default: 'euler'.
         point_type : str, {'vector', 'matrix'}
             Type of representation used for points.
+            Optional, default: None.
 
         Returns
         -------
-        exp : array-like, shape=[n_samples, dim]
+        exp : array-like, shape=[..., dim]
             Point on the manifold.
         """
-        tangent_vec = gs.to_ndarray(tangent_vec, to_ndim=2)
-        base_point = gs.to_ndarray(base_point, to_ndim=2)
         initial_state = (base_point, tangent_vec)
         flow, _ = integrate(
             self.geodesic_equation, initial_state, n_steps=n_steps, step=step)
-        return flow[-1]
+
+        exp = flow[-1]
+        return exp
 
     def log(self, point, base_point, n_steps=N_STEPS, step='euler'):
         """Compute logarithm map associated to the affine connection.
@@ -128,29 +141,30 @@ class Connection:
 
         Parameters
         ----------
-        point : array-like, shape=[n_samples, dim]
+        point : array-like, shape=[..., dim]
             Point on the manifold.
-        base_point : array-like, shape=[n_samples, dim]
+        base_point : array-like, shape=[..., dim]
             Point on the manifold.
         n_steps : int
-            The number of discrete time steps to take in the integration.
+            Number of discrete time steps to take in the integration.
+            Optional, default: N_STEPS.
         step : str, {'euler', 'rk4'}
-            The numerical scheme to use for integration.
+            Numerical scheme to use for integration.
+            Optional, default: 'euler'.
 
         Returns
         -------
-        tangent_vec : array-like, shape=[n_samples, dim]
+        tangent_vec : array-like, shape=[..., dim]
             Tangent vector at the base point.
         """
-        point = gs.to_ndarray(point, to_ndim=2)
-        base_point = gs.to_ndarray(base_point, to_ndim=2)
-        n_samples = len(base_point)
+        n_samples = geomstats.vectorization.get_n_points(
+            base_point, point_type='vector')
 
         def objective(velocity):
             """Define the objective function."""
             velocity = velocity.reshape(base_point.shape)
             delta = self.exp(velocity, base_point, n_steps, step) - point
-            loss = 1. / self.dim * gs.sum(delta ** 2, axis=1)
+            loss = 1. / self.dim * gs.sum(delta ** 2, axis=-1)
             return 1. / n_samples * gs.sum(loss)
 
         objective_grad = autograd.elementwise_grad(objective)
@@ -177,11 +191,11 @@ class Connection:
 
         Parameters
         ----------
-        base_point : array-like, shape=[n_samples, dim]
+        base_point : array-like, shape=[..., dim]
             Point on the manifold, from which to transport.
-        next_point : array-like, shape=[n_samples, dim]
+        next_point : array-like, shape=[..., dim]
             Point on the manifold, to transport to.
-        base_shoot : array-like, shape=[n_samples, dim]
+        base_shoot : array-like, shape=[..., dim]
             Point on the manifold, end point of the geodesics starting
             from the base point with initial speed to be transported.
         return_geodesics : bool, optional (defaults to False)
@@ -191,14 +205,14 @@ class Connection:
         Returns
         -------
         next_step : dict of array-like and callable with following keys:
-            next_tangent_vec : array-like, shape=[n_samples, dim]
+            next_tangent_vec : array-like, shape=[..., dim]
                 Tangent vector at end point.
-            end_point : array-like, shape=[n_samples, dim]
+            end_point : array-like, shape=[..., dim]
                 Point on the manifold, closes the geodesic parallelogram of the
                 construction.
             geodesics : list of callable, len=3 (only if
             `return_geodesics=True`)
-                The three geodesics of the construction.
+                Three geodesics of the construction.
 
         References
         ----------
@@ -255,22 +269,23 @@ class Connection:
 
         Parameters
         ----------
-        base_point : array-like, shape=[n_samples, dim]
+        base_point : array-like, shape=[..., dim]
             Point on the manifold, from which to transport.
-        next_point : array-like, shape=[n_samples, dim]
+        next_point : array-like, shape=[..., dim]
             Point on the manifold, to transport to.
-        base_shoot : array-like, shape=[n_samples, dim]
+        base_shoot : array-like, shape=[..., dim]
             Point on the manifold, end point of the geodesics starting
             from the base point with initial speed to be transported.
-        return_geodesics : bool, optional (defaults to False)
+        return_geodesics : bool
             Whether to return points computed along each geodesic of the
             construction.
+            Optional, default: False.
 
         Returns
         -------
-        transported_tangent_vector : array-like, shape=[n_samples, dim]
+        transported_tangent_vector : array-like, shape=[..., dim]
             Tangent vector at end point.
-        end_point : array-like, shape=[n_samples, dim]
+        end_point : array-like, shape=[..., dim]
             Point on the manifold, closes the geodesic parallelogram of the
             construction.
 
@@ -338,24 +353,26 @@ class Connection:
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[n_samples, dim]
+        tangent_vec_a : array-like, shape=[..., dim]
             Tangent vector at base point to transport.
-        tangent_vec_b : array-like, shape=[n_samples, dim]
+        tangent_vec_b : array-like, shape=[..., dim]
             Tangent vector at base point, initial speed of the geodesic along
             which to transport.
-        base_point : array-like, shape=[n_samples, dim]
+        base_point : array-like, shape=[..., dim]
             Point on the manifold, initial position of the geodesic along
             which to transport.
         n_steps : int
             The number of pole ladder steps.
+            Optional, default: 1.
         step : str, {'pole', 'schild'}
             The scheme to use for the construction of the ladder at each step.
+            Optoinal, default: 'pole'.
         **single_step_kwargs : keyword arguments for the step functions
 
         Returns
         -------
         ladder : dict of array-like and callable with following keys
-            transported_tangent_vector : array-like, shape=[n_samples, dim]
+            transported_tangent_vector : array-like, shape=[..., dim]
                 Approximation of the parallel transport of tangent vector a.
             trajectory : list of list of callable, len=n_steps
                 List of lists containing the geodesics of the
@@ -379,10 +396,10 @@ class Connection:
         methods = {'pole': self._pole_ladder_step,
                    'schild': self._schild_ladder_step}
         single_step = methods[step]
-        base_shoot = self.exp(base_point=current_point,
-                              tangent_vec=next_tangent_vec)
+        base_shoot = self.exp(
+            base_point=current_point, tangent_vec=next_tangent_vec)
         trajectory = []
-        for i_point in range(0, n_steps):
+        for i_point in range(n_steps):
             frac_tangent_vector_b = (i_point + 1) / n_steps * tangent_vec_b
             next_point = self.exp(
                 base_point=base_point,
@@ -405,7 +422,7 @@ class Connection:
 
         Parameters
         ----------
-        base_point: array-like, shape=[n_samples, dim]
+        base_point: array-like, shape=[..., dim]
             Point on the manifold.
         """
         raise NotImplementedError(
@@ -413,7 +430,7 @@ class Connection:
 
     def geodesic(self, initial_point,
                  end_point=None, initial_tangent_vec=None,
-                 point_type='vector'):
+                 point_type=None):
         """Generate parameterized function for the geodesic curve.
 
         Geodesic curve defined by either:
@@ -422,47 +439,55 @@ class Connection:
 
         Parameters
         ----------
-        initial_point : array-like, shape=[n_samples, dim]
+        initial_point : array-like, shape=[..., dim]
             Point on the manifold, initial point of the geodesic.
-        end_point : array-like, shape=[n_samples, dim], optional
+        end_point : array-like, shape=[..., dim], optional
             Point on the manifold, end point of the geodesic. If None,
             an initial tangent vector must be given.
-        initial_tangent_vec : array-like, shape=[n_samples, dim],
-            optional
+        initial_tangent_vec : array-like, shape=[..., dim],
             Tangent vector at base point, the initial speed of the geodesics.
+            Optional, default: None.
             If None, an end point must be given and a logarithm is computed.
         point_type : str, {'vector', 'matrix'}
-            The type of point.
+            Point type.
+            Optional, default: 'vector'.
 
         Returns
         -------
         path : callable
-            The time parameterized geodesic curve.
+            Time parameterized geodesic curve. If a batch of initial
+            conditions is passed, the output array's first dimension
+            represents time, and the second corresponds to the different
+            initial conditions.
         """
-        point_ndim = 1
-        if point_type == 'matrix':
-            point_ndim = 2
-
-        initial_point = gs.to_ndarray(
-            initial_point, to_ndim=point_ndim + 1)
+        if point_type is None:
+            point_type = self.default_point_type
+        geomstats.errors.check_parameter_accepted_values(
+            point_type, 'point_type', ['vector', 'matrix'])
 
         if end_point is None and initial_tangent_vec is None:
             raise ValueError('Specify an end point or an initial tangent '
                              'vector to define the geodesic.')
         if end_point is not None:
-            end_point = gs.to_ndarray(end_point, to_ndim=point_ndim + 1)
             shooting_tangent_vec = self.log(
-                point=end_point,
-                base_point=initial_point)
+                point=end_point, base_point=initial_point)
             if initial_tangent_vec is not None:
                 if not gs.allclose(shooting_tangent_vec, initial_tangent_vec):
                     raise RuntimeError(
                         'The shooting tangent vector is too'
-                        ' far from the initial tangent vector.')
+                        ' far from the input initial tangent vector.')
             initial_tangent_vec = shooting_tangent_vec
-        initial_tangent_vec = gs.array(initial_tangent_vec)
-        initial_tangent_vec = gs.to_ndarray(
-            initial_tangent_vec, to_ndim=point_ndim + 1)
+
+        if point_type == 'vector':
+            initial_point = gs.to_ndarray(initial_point, to_ndim=2)
+            initial_tangent_vec = gs.to_ndarray(
+                initial_tangent_vec, to_ndim=2)
+
+        else:
+            initial_point = gs.to_ndarray(initial_point, to_ndim=3)
+            initial_tangent_vec = gs.to_ndarray(
+                initial_tangent_vec, to_ndim=3)
+        n_initial_conditions = initial_tangent_vec.shape[0]
 
         def path(t):
             """Generate parameterized function for geodesic curve.
@@ -472,33 +497,22 @@ class Connection:
             t : array-like, shape=[n_points,]
                 Times at which to compute points of the geodesics.
             """
-            t = gs.array(t)
-            t = gs.cast(t, gs.float32)
+            t = gs.array(t, gs.float32)
             t = gs.to_ndarray(t, to_ndim=1)
-            t = gs.to_ndarray(t, to_ndim=2, axis=1)
-            new_initial_point = gs.to_ndarray(
-                initial_point,
-                to_ndim=point_ndim + 1)
-            new_initial_tangent_vec = gs.to_ndarray(
-                initial_tangent_vec,
-                to_ndim=point_ndim + 1)
-
             if point_type == 'vector':
                 tangent_vecs = gs.einsum(
-                    'il,nk->ik',
-                    t,
-                    new_initial_tangent_vec)
-            elif point_type == 'matrix':
+                    'i,...k->...ik', t, initial_tangent_vec)
+            else:
                 tangent_vecs = gs.einsum(
-                    'il,nkm->ikm',
-                    t,
-                    new_initial_tangent_vec)
+                    'i,...kl->...ikl', t, initial_tangent_vec)
 
-            point_at_time_t = self.exp(
-                tangent_vec=tangent_vecs,
-                base_point=new_initial_point)
-            return point_at_time_t
+            points_at_time_t = [
+                self.exp(tv, pt) for tv,
+                pt in zip(tangent_vecs, initial_point)]
+            points_at_time_t = gs.stack(points_at_time_t, axis=1)
 
+            return points_at_time_t[:, 0] if n_initial_conditions == 1 else \
+                points_at_time_t
         return path
 
     def torsion(self, base_point):
@@ -506,7 +520,7 @@ class Connection:
 
         Parameters
         ----------
-        base_point: array-like, shape=[n_samples, dim]
+        base_point: array-like, shape=[..., dim]
             Point on the manifold.
         """
         raise NotImplementedError(
