@@ -12,6 +12,12 @@ class EmbeddedManifold(Manifold):
         Dimension of the embedded manifold.
     embedding_manifold : Manifold
         Embedding manifold.
+    default_point_type : str, {'vector', 'matrix'}
+        Point type.
+        Optional, default: 'vector'.
+    default_coords_type : str, {'intrinsic', 'extrinsic', etc}
+        Coordinate type.
+        Optional, default: 'intrinsic'.
     """
 
     def __init__(self, dim, embedding_manifold, default_point_type='vector',
@@ -26,8 +32,13 @@ class EmbeddedManifold(Manifold):
 
         Parameters
         ----------
-        point_intrinsic : array-like, shape=[n_samples, dim]
+        point_intrinsic : array-like, shape=[..., dim]
             Point in the embedded manifold in intrinsic coordinates.
+
+        Returns
+        -------
+        point_extrinsic : array-like, shape=[..., dim_embedding]
+            Point in the embedded manifold in extrinsic coordinates.
         """
         raise NotImplementedError(
             'intrinsic_to_extrinsic_coords is not implemented.')
@@ -37,9 +48,14 @@ class EmbeddedManifold(Manifold):
 
         Parameters
         ----------
-        point_extrinsic : array-like, shape=[n_samples, dim_embedding]
+        point_extrinsic : array-like, shape=[..., dim_embedding]
             Point in the embedded manifold in extrinsic coordinates,
             i. e. in the coordinates of the embedding manifold.
+
+        Returns
+        -------
+        point_intrinsic : array-lie, shape=[..., dim]
+            Point in the embedded manifold in intrinsic coordinates.
         """
         raise NotImplementedError(
             'extrinsic_to_intrinsic_coords is not implemented.')
@@ -49,21 +65,12 @@ class EmbeddedManifold(Manifold):
 
         Parameters
         ----------
-        point : array-like, shape=[n_samples, dim_embedding]
-            Point in embedding manifold
+        point : array-like, shape=[..., dim_embedding]
+            Point in embedding manifold.
+
+        Returns
+        -------
+            Projected point.
         """
         raise NotImplementedError(
             'projection is not implemented.')
-
-    def projection_to_tangent_space(self, vector, base_point):
-        """Project a vector to a tangent space of the embedded manifold.
-
-        Parameters
-        ----------
-        vector : array-like, shape=[n_samples, dim_embedding]
-            Vector at the tangent space of the embedding manifold.
-        base_point : array-like, shape=[n_samples, dim_embedding]
-            Point on the embedded manifold, in extrinsic coordinates.
-        """
-        raise NotImplementedError(
-            'projection_to_tangent_space is not implemented.')
