@@ -1,29 +1,49 @@
 """Module exposing the GeneralLinear group class."""
 
+from itertools import product
+
 import geomstats.backend as gs
 from geomstats.geometry.matrices import Matrices
 
 
 class GeneralLinear(Matrices):
-    """Class for the general linear group GL(n)."""
+    """Class for the general linear group GL(n).
 
-    def __init__(self, n):
+    Parameters
+    ----------
+    n : int
+        Integer representing the shape of the matrices: n x n.
+    """
+
+    def __init__(self, n, **kwargs):
         Matrices.__init__(self, n, n)
 
         self.n = n
 
     def belongs(self, point):
-        """Test if a matrix is invertible and of the right size."""
-        point = gs.to_ndarray(point, to_ndim=3)
-        _, mat_dim_1, mat_dim_2 = point.shape
+        """Check if a matrix is invertible and of the right shape.
+
+        Parameters
+        ----------
+        point : array-like, shape=[..., n, n]
+            Matrix to be checked.
+
+        Returns
+        -------
+        belongs : array-like, shape=[...,]
+            Boolean denoting if point is in GL(n).
+        """
+        point_shape = point.shape
+        mat_dim_1, mat_dim_2 = point_shape[-2], point_shape[-1]
         det = gs.linalg.det(point)
         return gs.logical_and(
-            mat_dim_1 == self.n and mat_dim_2 == self.n, gs.where(
-                det != 0., gs.array(True), gs.array(False)))
+            mat_dim_1 == self.n and mat_dim_2 == self.n,
+            det != 0.)
 
-    def identity(self):
+    def get_identity(self):
         """Return the identity matrix."""
         return gs.eye(self.n, self.n)
+    identity = property(get_identity)
 
     @classmethod
     def compose(cls, *args):
@@ -31,25 +51,41 @@ class GeneralLinear(Matrices):
         return cls.mul(*args)
 
     @staticmethod
-    def inv(point):
-        """Return the inverse of a matrix."""
+    def inverse(point):
+        """Return the inverse of a matrix.
+
+        Parameters
+        ----------
+        point : array-like, shape=[..., n, n]
+            Matrix to be inverted.
+        """
         return gs.linalg.inv(point)
 
+    def _replace_values(self, samples, new_samples, indcs):
+        """Replace samples with new samples at specific indices."""
+        replaced_indices = [
+            i for i, is_replaced in enumerate(indcs) if is_replaced]
+        value_indices = list(
+            product(replaced_indices, range(self.n), range(self.n)))
+        return gs.assignment(samples, gs.flatten(new_samples), value_indices)
+
     def random_uniform(self, n_samples=1, tol=1e-6):
         """Sample in GL(n) from the uniform distribution.
 
         Parameters
         ----------
-        n_samples : int, optional
+        n_samples : int
             Number of samples.
-        tol: float, optional
+            Optional, default: 1.
+        tol: float
             Threshold for the absolute value of the determinant of the
             returned matrix.
+            Optional, default: 1e-6.
 
         Returns
         -------
-        samples : array-like, shape=[n_samples, n, n]
-            Points sampled on GL(n).
+        samples : array-like, shape=[..., n, n]
+            Point sampled on GL(n).
         """
         samples = gs.random.rand(n_samples, self.n, self.n)
         while True:
@@ -58,7 +94,8 @@ class GeneralLinear(Matrices):
             num_bad_samples = gs.sum(indcs)
             if num_bad_samples == 0:
                 break
-            samples[indcs, :] = gs.random.rand(num_bad_samples, self.n, self.n)
+            new_samples = gs.random.rand(num_bad_samples, self.n, self.n)
+            samples = self._replace_values(samples, new_samples, indcs)
         if n_samples == 1:
             samples = gs.squeeze(samples, axis=0)
         return samples
@@ -82,40 +119,43 @@ class GeneralLinear(Matrices):
 
         Parameters
         ----------
-        tangent_vec :   array-like, shape=[..., n, n]
-        base_point :    array-like, shape=[..., n, n]
-            Defaults to identity.
+        tangent_vec : array-like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array-like, shape=[..., n, n]
+            Base point.
+            Optional, defaults to identity if None.
 
         Returns
         -------
-        point :         array-like, shape=[..., n, n]
-            The left multiplication of `exp(algebra_mat)` with
-            `base_point`.
+        point : array-like, shape=[..., n, n]
+            Left multiplication of `exp(algebra_mat)` with `base_point`.
         """
         expm = gs.linalg.expm
         if base_point is None:
             return expm(tangent_vec)
-        lie_algebra_vec = cls.mul(cls.inv(base_point), tangent_vec)
+        lie_algebra_vec = cls.mul(cls.inverse(base_point), tangent_vec)
         return cls.mul(base_point, cls.exp(lie_algebra_vec))
 
     @classmethod
     def log(cls, point, base_point=None):
         r"""
-        Calculate a left-invariant vector field bringing base_point to point.
+        Compute a left-invariant vector field bringing base_point to point.
 
         The output is a vector of the tangent space at base_point, so not a Lie
         algebra element if it is not the identity.
 
         Parameters
         ----------
-        point :         array-like, shape=[..., n, n]
-        base_point :    array-like, shape=[..., n, n]
-            Defaults to identity.
+        point : array-like, shape=[..., n, n]
+            Point.
+        base_point : array-like, shape=[..., n, n]
+            Base point.
+            Optional, defaults to identity if None.
 
         Returns
         -------
-        tangent_vec :   array-like, shape=[..., n, n]
-            A matrix such that `exp(tangent_vec, base_point) = point`.
+        tangent_vec : array-like, shape=[..., n, n]
+            Matrix such that `exp(tangent_vec, base_point) = point`.
 
         Notes
         -----
@@ -129,7 +169,7 @@ class GeneralLinear(Matrices):
         logm = gs.linalg.logm
         if base_point is None:
             return logm(point)
-        lie_algebra_vec = logm(cls.mul(cls.inv(base_point), point))
+        lie_algebra_vec = logm(cls.mul(cls.inverse(base_point), point))
         return cls.mul(base_point, lie_algebra_vec)
 
     @classmethod
@@ -142,12 +182,13 @@ class GeneralLinear(Matrices):
         point : array-like, shape=[n, n]
             Target point.
         base_point : array-like, shape=[n, n], optional
-            Base point. Defaults to identity.
+            Base point.
+            Optional, defaults to identity if None.
 
         Returns
         -------
         path : callable
-            The one-parameter orbit.
+            One-parameter orbit.
             Satisfies `path(0) = base_point` and `path(1) = point`.
 
         Notes
@@ -168,9 +209,8 @@ class GeneralLinear(Matrices):
         -------------
         Return a collection of trajectories (4-D array)
         from a collection of input matrices (3-D array).
-
-        # TODO(nina): Will work when expm gets properly 4-D vectorized.
         """
+        # TODO (nina): Will work when expm gets properly 4-D vectorized.
         tangent_vec = cls.log(point, base_point)
 
         def path(time):
