@@ -3,7 +3,7 @@
 import logging
 
 import geomstats.backend as gs
-import geomstats.error
+import geomstats.errors
 from geomstats.geometry.general_linear import GeneralLinear
 from geomstats.geometry.matrices import Matrices
 from geomstats.geometry.riemannian_metric import RiemannianMetric
@@ -21,22 +21,25 @@ class InvariantMetric(RiemannianMetric):
     Parameters
     ----------
     group : LieGroup
-        The group to equip with the invariant metric
+        Group to equip with the invariant metric
     inner_product_mat_at_identity : array-like, shape=[dim, dim]
-        The matrix that defines the metric at identity.
+        Matrix that defines the metric at identity.
+        Optional, defaults to identity matrix if None.
     left_or_right : str, {'left', 'right'}
         Wether to use a left or right invariant metric.
+        Optional, default: 'left'.
     """
 
     def __init__(self, group,
                  inner_product_mat_at_identity=None,
-                 left_or_right='left'):
+                 left_or_right='left', **kwargs):
+        super(InvariantMetric, self).__init__(dim=group.dim, **kwargs)
 
         self.group = group
         if inner_product_mat_at_identity is None:
             inner_product_mat_at_identity = gs.eye(self.group.dim)
 
-        geomstats.error.check_parameter_accepted_values(
+        geomstats.errors.check_parameter_accepted_values(
             left_or_right, 'left_or_right', ['left', 'right'])
 
         eigenvalues = gs.linalg.eigvalsh(inner_product_mat_at_identity)
@@ -56,24 +59,23 @@ class InvariantMetric(RiemannianMetric):
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[n_samples, dim]
+        tangent_vec_a : array-like, shape=[..., dim]
             First tangent vector at identity.
-        tangent_vec_b : array-like, shape=[n_samples, dim]
+        tangent_vec_b : array-like, shape=[..., dim]
             Second tangent vector at identity.
 
         Returns
         -------
-        inner_prod : array-like, shape=[n_samples, dim]
+        inner_prod : array-like, shape=[..., dim]
             Inner-product of the two tangent vectors.
         """
-        geomstats.error.check_parameter_accepted_values(
+        geomstats.errors.check_parameter_accepted_values(
             self.group.default_point_type,
             'default_point_type',
             ['vector', 'matrix'])
 
         if self.group.default_point_type == 'vector':
             inner_product_mat_at_identity = self.inner_product_mat_at_identity
-
             inner_prod = gs.einsum(
                 '...i,...ij->...j',
                 tangent_vec_a,
@@ -82,7 +84,7 @@ class InvariantMetric(RiemannianMetric):
                 '...j,...j->...', inner_prod, tangent_vec_b)
 
         else:
-            # TODO(nguigs): allow for diagonal metric_matrices
+            # TODO (nguigs): allow for diagonal metric_matrices
             logging.warning(
                 'Only the canonical inner product -Frobenius inner product-'
                 ' is implemented for Lie groups whose elements are represented'
@@ -100,16 +102,17 @@ class InvariantMetric(RiemannianMetric):
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[n_samples, dim]
+        tangent_vec_a : array-like, shape=[..., dim]
             First tangent vector at base_point.
-        tangent_vec_b : array-like, shape=[n_samples, dim]
+        tangent_vec_b : array-like, shape=[..., dim]
             Second tangent vector at base_point.
-        base_point : array-like, shape=[n_samples, dim], optional
-            Point in the group (the default is identity).
+        base_point : array-like, shape=[..., dim]
+            Point in the group.
+            Optional, defaults to identity if None.
 
         Returns
         -------
-        inner_prod : array-like, shape=[n_samples, dim]
+        inner_prod : array-like, shape=[..., dim]
             Inner-product of the two tangent vectors.
         """
         if base_point is None:
@@ -121,18 +124,10 @@ class InvariantMetric(RiemannianMetric):
                 tangent_vec_b,
                 base_point)
 
-        jacobian = self.group.jacobian_translation(base_point)
-        inv_jacobian = self.group.inverse(jacobian)
-        if self.left_or_right == 'left':
-            tangent_vec_a_at_id = self.group.compose(
-                inv_jacobian, tangent_vec_a)
-            tangent_vec_b_at_id = self.group.compose(
-                inv_jacobian, tangent_vec_b)
-        elif self.left_or_right == 'right':
-            tangent_vec_a_at_id = self.group.compose(
-                tangent_vec_a, inv_jacobian)
-            tangent_vec_b_at_id = self.group.compose(
-                tangent_vec_b, inv_jacobian)
+        tangent_translation = self.group.tangent_translation_map(
+            base_point, left_or_right=self.left_or_right, inverse=True)
+        tangent_vec_a_at_id = tangent_translation(tangent_vec_a)
+        tangent_vec_b_at_id = tangent_translation(tangent_vec_b)
         inner_prod = self.inner_product_at_identity(
             tangent_vec_a_at_id, tangent_vec_b_at_id)
         return inner_prod
@@ -142,13 +137,13 @@ class InvariantMetric(RiemannianMetric):
 
         Parameters
         ----------
-        base_point : array-like, shape=[n_samples, dim], optional
+        base_point : array-like, shape=[..., dim], optional
             Point in the group (the default is identity).
 
         Returns
         -------
-        metric_mat : array-like, shape=[n_samples, dim, dim]
-            The metric matrix at base_point.
+        metric_mat : array-like, shape=[..., dim, dim]
+            Metric matrix at base_point.
         """
         if self.group.default_point_type == 'matrix':
             raise NotImplementedError(
@@ -157,13 +152,13 @@ class InvariantMetric(RiemannianMetric):
 
         if base_point is None:
             base_point = self.group.identity
-        base_point = self.group.regularize(base_point)
+        else:
+            base_point = self.group.regularize(base_point)
 
         jacobian = self.group.jacobian_translation(
-            point=base_point,
-            left_or_right=self.left_or_right)
+            point=base_point, left_or_right=self.left_or_right)
 
-        inv_jacobian = GeneralLinear.inv(jacobian)
+        inv_jacobian = GeneralLinear.inverse(jacobian)
         inv_jacobian_transposed = Matrices.transpose(inv_jacobian)
 
         metric_mat = gs.einsum(
@@ -185,12 +180,12 @@ class InvariantMetric(RiemannianMetric):
 
         Parameters
         ----------
-        tangent_vec : array-like, shape=[n_samples, dim]
+        tangent_vec : array-like, shape=[..., dim]
             Tangent vector at identity.
 
         Returns
         -------
-        exp : array-like, shape=[n_samples, dim]
+        exp : array-like, shape=[..., dim]
             Point in the group.
         """
         tangent_vec = self.group.regularize_tangent_vec_at_identity(
@@ -210,12 +205,12 @@ class InvariantMetric(RiemannianMetric):
 
         Parameters
         ----------
-        tangent_vec : array-like, shape=[n_samples, dim]
+        tangent_vec : array-like, shape=[..., dim]
             Tangent vector at identity.
 
         Returns
         -------
-        exp : array-like, shape=[n_samples, dim]
+        exp : array-like, shape=[..., dim]
             Point in the group.
         """
         if self.left_or_right == 'left':
@@ -233,14 +228,15 @@ class InvariantMetric(RiemannianMetric):
 
         Parameters
         ----------
-        tangent_vec : array-like, shape=[n_samples, dim]
+        tangent_vec : array-like, shape=[..., dim]
             Tangent vector at a base point.
-        base_point : array-like, shape=[n_samples, dim]
+        base_point : array-like, shape=[..., dim]
             Point in the group.
+            Optional, defaults to identity if None.
 
         Returns
         -------
-        exp : array-like, shape=[n_samples, dim]
+        exp : array-like, shape=[..., dim]
             Point in the group equal to the Riemannian exponential
             of tangent_vec at the base point.
         """
@@ -253,12 +249,10 @@ class InvariantMetric(RiemannianMetric):
         if gs.allclose(base_point, identity):
             return self.exp_from_identity(tangent_vec)
 
-        jacobian = self.group.jacobian_translation(
-            point=base_point, left_or_right=self.left_or_right)
-        inv_jacobian = gs.linalg.inv(jacobian)
-        inv_jacobian_transposed = Matrices.transpose(inv_jacobian)
-        tangent_vec_at_id = gs.einsum(
-            '...i,...ij->...j', tangent_vec, inv_jacobian_transposed)
+        tangent_vec_at_id = self.group.tangent_translation_map(
+            point=base_point,
+            left_or_right=self.left_or_right,
+            inverse=True)(tangent_vec)
         exp_from_id = self.exp_from_identity(tangent_vec_at_id)
 
         if self.left_or_right == 'left':
@@ -283,18 +277,18 @@ class InvariantMetric(RiemannianMetric):
 
         Parameters
         ----------
-        point : array-like, shape=[n_samples, dim]
+        point : array-like, shape=[..., dim]
             Point in the group.
 
         Returns
         -------
-        log : array-like, shape=[n_samples, dim]
+        log : array-like, shape=[..., dim]
             Tangent vector at the identity equal to the Riemannian logarithm
             of point at the identity.
         """
         point = self.group.regularize(point)
         inner_prod_mat = self.inner_product_mat_at_identity
-        inv_inner_prod_mat = GeneralLinear.inv(inner_prod_mat)
+        inv_inner_prod_mat = GeneralLinear.inverse(inner_prod_mat)
         sqrt_inv_inner_prod_mat = gs.linalg.sqrtm(inv_inner_prod_mat)
         log = gs.einsum('...i,...ij->...j', point, sqrt_inv_inner_prod_mat)
         log = self.group.regularize_tangent_vec_at_identity(
@@ -306,12 +300,12 @@ class InvariantMetric(RiemannianMetric):
 
         Parameters
         ----------
-        point : array-like, shape=[n_samples, dim]
+        point : array-like, shape=[..., dim]
             Point in the group.
 
         Returns
         -------
-        log : array-like, shape=[n_samples, dim]
+        log : array-like, shape=[..., dim]
             Tangent vector at the identity equal to the Riemannian logarithm
             of point at the identity.
         """
@@ -331,15 +325,15 @@ class InvariantMetric(RiemannianMetric):
 
         Parameters
         ----------
-        point : array-like, shape=[n_samples, dim]
+        point : array-like, shape=[..., dim]
             Point in the group.
-        base_point : array-like, shape=[n_samples, dim], optional
+        base_point : array-like, shape=[..., dim], optional
             Point in the group, from which to compute the log,
             (the default is identity).
 
         Returns
         -------
-        log : array-like, shape=[n_samples, dim]
+        log : array-like, shape=[..., dim]
             Tangent vector at the base point equal to the Riemannian logarithm
             of point at the base point.
         """
@@ -363,12 +357,68 @@ class InvariantMetric(RiemannianMetric):
                 point, self.group.inverse(base_point))
 
         log_from_id = self.log_from_identity(point_near_id)
-
-        jacobian = self.group.jacobian_translation(
-            base_point, left_or_right=self.left_or_right)
-
-        log = gs.einsum(
-            '...j,...jk->...k',
-            log_from_id,
-            Matrices.transpose(jacobian))
+        log = self.group.tangent_translation_map(
+            base_point, left_or_right=self.left_or_right)(log_from_id)
         return log
+
+
+class BiInvariantMetric(InvariantMetric):
+    """Class for bi-invariant metrics which exist on Lie groups.
+
+    Compact Lie groups and direct products of compact Lie groups with vector
+    spaces admit bi-invariant metrics. Products Lie groups are not
+    implemented. Other groups such as SE(3) admit bi-invariant pseudo-metrics.
+
+    Parameters
+    ----------
+    group : LieGroup
+        The group to equip with the bi-invariant metric
+    """
+
+    def __init__(self, group):
+        super(BiInvariantMetric, self).__init__(
+            group=group, inner_product_mat_at_identity=gs.eye(group.dim),
+            default_point_type=group.default_point_type)
+        cond = (
+            'SpecialOrthogonal' not in group.__str__()
+            and 'SO' not in group.__str__()
+            and 'SpecialOrthogonal3' not in group.__str__())
+        # TODO (nguigs): implement it for SE(3)
+        if cond:
+            raise ValueError(
+                'The bi-invariant metric is only implemented for SO(n)')
+
+    def exp_from_identity(self, tangent_vec):
+        """Compute Riemannian exponential of tangent vector from the identity.
+
+        For a bi-invariant metric, this corresponds to the group exponential.
+
+        Parameters
+        ----------
+        tangent_vec : array-like, shape=[..., {dim, [n, n]}]
+            Tangent vector at identity.
+
+        Returns
+        -------
+        exp : array-like, shape=[..., {dim, [n, n]}]
+            Point in the group.
+        """
+        return self.group.exp(tangent_vec)
+
+    def log_from_identity(self, point):
+        """Compute Riemannian logarithm of a point wrt the identity.
+
+        For a bi-invariant metric this corresponds to the group logarithm.
+
+        Parameters
+        ----------
+        point : array-like, shape=[..., {dim, [n, n]}]
+            Point in the group.
+
+        Returns
+        -------
+        log : array-like, shape=[..., {dim, [n, n]}]
+            Tangent vector at the identity equal to the Riemannian logarithm
+            of point at the identity.
+        """
+        return self.group.log(point)
