@@ -3,7 +3,7 @@
 import joblib
 
 import geomstats.backend as gs
-import geomstats.error
+import geomstats.errors
 import geomstats.vectorization
 from geomstats.geometry.manifold import Manifold
 from geomstats.geometry.product_riemannian_metric import \
@@ -18,12 +18,12 @@ class ProductManifold(Manifold):
     same dimension, but the list of manifolds needs to be provided.
 
     By default, a point is represented by an array of shape:
-    [n_samples, dim_1 + ... + dim_n_manifolds]
+    [..., dim_1 + ... + dim_n_manifolds]
     where n_manifolds is the number of manifolds in the product.
     This type of representation is called 'vector'.
 
     Alternatively, a point can be represented by an array of shape:
-    [n_samples, n_manifolds, dim] if the n_manifolds have same dimension dim.
+    [..., n_manifolds, dim] if the n_manifolds have same dimension dim.
     This type of representation is called `matrix`.
 
     Parameters
@@ -32,12 +32,16 @@ class ProductManifold(Manifold):
         List of manifolds in the product.
     default_point_type : str, {'vector', 'matrix'}
         Default representation of points.
+        Optional, default: 'vector'.
+    n_jobs : int
+        Number of jobs for parallel computing.
+        Optional, default: 1.
     """
 
-    # FIXME(nguigs): This only works for 1d points
+    # FIXME (nguigs): This only works for 1d points
 
     def __init__(self, manifolds, default_point_type='vector', n_jobs=1):
-        geomstats.error.check_parameter_accepted_values(
+        geomstats.errors.check_parameter_accepted_values(
             default_point_type, 'default_point_type', ['vector', 'matrix'])
 
         self.dims = [manifold.dim for manifold in manifolds]
@@ -76,27 +80,29 @@ class ProductManifold(Manifold):
 
         Parameters
         ----------
-        point : array-like, shape=[n_samples, dim]
-                           or shape=[n_samples, dim_2, dim_2]
+        point : array-like, shape=[..., {dim, [dim_2, dim_2]}]
             Point.
         point_type : str, {'vector', 'matrix'}
             Representation of point.
+            Optional, default: None.
 
         Returns
         -------
-        belongs : array-like, shape=[n_samples, 1]
-            Array of booleans evaluating if the corresponding points
-            belong to the manifold.
+        belongs : array-like, shape=[...,]
+            Boolean evaluating if the point belongs to the manifold.
         """
         if point_type is None:
             point_type = self.default_point_type
+        geomstats.errors.check_parameter_accepted_values(
+            point_type, 'point_type', ['vector', 'matrix'])
+
         if point_type == 'vector':
             intrinsic = self.metric.is_intrinsic(point)
             belongs = self._iterate_over_manifolds(
                 'belongs', {'point': point}, intrinsic)
             belongs = gs.stack(belongs, axis=1)
 
-        elif point_type == 'matrix':
+        else:
             belongs = gs.stack([
                 space.belongs(point[:, i]) for i, space in enumerate(
                     self.manifolds)],
@@ -112,21 +118,20 @@ class ProductManifold(Manifold):
 
         Parameters
         ----------
-        point : array-like, shape=[n_samples, dim]
-                           or shape=[n_samples, dim_2, dim_2]
+        point : array-like, shape=[..., {dim, [dim_2, dim_2]}]
             Point to be regularized.
         point_type : str, {'vector', 'matrix'}
             Representation of point.
+            Optional, default: None.
 
         Returns
         -------
-        regularized_point : array-like, shape=[n_samples, dim]
-                            or shape=[n_samples, dim_2, dim_2]
+        regularized_point : array-like, shape=[..., {dim, [dim_2, dim_2]}]
             Point in the manifold's canonical representation.
         """
         if point_type is None:
             point_type = self.default_point_type
-        geomstats.error.check_parameter_accepted_values(
+        geomstats.errors.check_parameter_accepted_values(
             point_type, 'point_type', ['vector', 'matrix'])
 
         if point_type == 'vector':
@@ -150,15 +155,16 @@ class ProductManifold(Manifold):
             Number of samples.
         point_type : str, {'vector', 'matrix'}
             Representation of point.
+            Optional, default: None.
 
         Returns
         -------
-        samples : array-like, shape=[n_samples, dim + 1]
+        samples : array-like, shape=[..., dim + 1]
             Points sampled on the hypersphere.
         """
         if point_type is None:
             point_type = self.default_point_type
-        geomstats.error.check_parameter_accepted_values(
+        geomstats.errors.check_parameter_accepted_values(
             point_type, 'point_type', ['vector', 'matrix'])
 
         if point_type == 'vector':
@@ -169,8 +175,6 @@ class ProductManifold(Manifold):
                     data = gs.concatenate([data, samples], axis=-1)
             return data
 
-        point = [
-            space.random_uniform(n_samples)
-            for space in self.manifolds]
-        samples = gs.stack(point, axis=1)
+        point = [space.random_uniform(n_samples) for space in self.manifolds]
+        samples = gs.stack(point, axis=-2)
         return samples
