@@ -24,36 +24,69 @@ def loss(y_pred, y_true, metric):
 
     Parameters
     ----------
-    y_pred
-    y_true
-    metric
+    y_pred : array-like, shape=[..., dim]
+        Prediction.
+    y_true : array-like, shape=[..., dim]
+        Ground-truth.
+    metric : RiemannianMetric
+        Metric.
 
     Returns
     -------
-    loss
-
+    sq_dist : array-like, shape=[...,]
+        Loss, i.e. the squared distance.
     """
     sq_dist = metric.squared_dist(y_pred, y_true)
     return sq_dist
 
 
 def grad(y_pred, y_true, metric):
-    """Closed-form for the gradient of the loss function."""
+    """Closed-form for the gradient of the loss function.
+
+    Parameters
+    ----------
+    y_pred : array-like, shape=[..., dim]
+        Prediction.
+    y_true : array-like, shape=[..., dim]
+        Ground-truth.
+    metric : RiemannianMetric
+        Metric.
+
+    Returns
+    -------
+    loss_grad : array-like, shape=[...,]
+        Gradient of the loss.
+
+    """
     tangent_vec = metric.log(base_point=y_pred, point=y_true)
     grad_vec = - 2. * tangent_vec
 
     inner_prod_mat = metric.inner_product_matrix(base_point=y_pred)
+    is_vectorized = inner_prod_mat.ndim == 3
+    axes = (0, 2, 1) if is_vectorized else (1, 0)
 
     loss_grad = gs.einsum(
-        'ni,nij->ni',
+        '...i,...ij->...i',
         grad_vec,
-        gs.transpose(inner_prod_mat, axes=(0, 2, 1)))
+        gs.transpose(inner_prod_mat, axes=axes))
 
     return loss_grad
 
 
 class RiemannianMetric(Connection):
-    """Class for Riemannian and pseudo-Riemannian metrics."""
+    """Class for Riemannian and pseudo-Riemannian metrics.
+
+    Parameters
+    ----------
+    dim : int
+        Dimension of the manifold.
+    signature : tuple
+        Signature of the metric.
+        Optional, default: None.
+    default_point_type : str, {'vector', 'matrix'}
+        Point type.
+        Optional, default: 'vector'.
+    """
 
     def __init__(self, dim, signature=None, default_point_type='vector'):
         super(RiemannianMetric, self).__init__(
@@ -65,7 +98,14 @@ class RiemannianMetric(Connection):
 
         Parameters
         ----------
-        base_point : array-like, shape=[n_samples, dim], optional
+        base_point : array-like, shape=[..., dim]
+            Base point.
+            Optional, default: None.
+
+        Returns
+        -------
+        mat : array-like, shape=[..., dim, dim]
+            Inner-product matrix.
         """
         raise NotImplementedError(
             'The computation of the inner product matrix'
@@ -76,7 +116,14 @@ class RiemannianMetric(Connection):
 
         Parameters
         ----------
-        base_point : array-like, shape=[n_samples, dim], optional
+        base_point : array-like, shape=[..., dim]
+            Base point.
+            Optional, default: None.
+
+        Returns
+        -------
+        mat : array-like, shape=[..., dim, dim]
+            Inverse of inner-product matrix.
         """
         metric_matrix = self.inner_product_matrix(base_point)
         cometric_matrix = gs.linalg.inv(metric_matrix)
@@ -87,7 +134,14 @@ class RiemannianMetric(Connection):
 
         Parameters
         ----------
-        base_point : array-like, shape=[n_samples, dim], optional
+        base_point : array-like, shape=[..., dim]
+            Base point.
+            Optional, default: None.
+
+        Returns
+        -------
+        mat : array-like, shape=[..., dim, dim]
+            Derivative of inverse of inner-product matrix.
         """
         metric_derivative = autograd.jacobian(self.inner_product_matrix)
         return metric_derivative(base_point)
@@ -97,12 +151,13 @@ class RiemannianMetric(Connection):
 
         Parameters
         ----------
-        base_point: array-like, shape=[n_samples, dim]
+        base_point: array-like, shape=[..., dim]
+            Base point.
 
         Returns
         -------
-        christoffels: array-like,
-                             shape=[n_samples, dim, dim, dim]
+        christoffels: array-like, shape=[..., dim, dim, dim]
+            Christoffel symbols.
         """
         cometric_mat_at_point = self.inner_product_inverse_matrix(base_point)
         metric_derivative_at_point = self.inner_product_derivative_matrix(
@@ -126,16 +181,18 @@ class RiemannianMetric(Connection):
 
         Parameters
         ----------
-        tangent_vec_a: array-like, shape=[n_samples, dim]
-                                   or shape=[1, dim]
-        tangent_vec_b: array-like, shape=[n_samples, dim]
-                                   or shape=[1, dim]
-        base_point: array-like, shape=[n_samples, dim]
-                                or shape=[1, dim]
+        tangent_vec_a: array-like, shape=[..., dim]
+            Tangent vector at base point.
+        tangent_vec_b: array-like, shape=[..., dim]
+            Tangent vector at base point.
+        base_point: array-like, shape=[..., dim]
+            Base point.
+            Optional, default: None.
 
         Returns
         -------
-        inner_product : array-like, shape=[n_samples,]
+        inner_product : array-like, shape=[...,]
+            Inner-product.
         """
         inner_prod_mat = self.inner_product_matrix(base_point)
         inner_prod_mat = gs.to_ndarray(inner_prod_mat, to_ndim=3)
@@ -156,12 +213,16 @@ class RiemannianMetric(Connection):
 
         Parameters
         ----------
-        vector : array-like, shape=[n_samples, dim]
-        base_point : array-like, shape=[n_samples, dim]
+        vector : array-like, shape=[..., dim]
+            Vector.
+        base_point : array-like, shape=[..., dim]
+            Base point.
+            Optional, default: None.
 
         Returns
         -------
-        sq_norm : array-like, shape=[n_samples,]
+        sq_norm : array-like, shape=[...,]
+            Squared norm.
         """
         sq_norm = self.inner_product(vector, vector, base_point)
         return sq_norm
@@ -177,12 +238,16 @@ class RiemannianMetric(Connection):
 
         Parameters
         ----------
-        vector : array-like, shape=[n_samples, dim]
-        base_point : array-like, shape=[n_samples, dim]
+        vector : array-like, shape=[..., dim]
+            Vector.
+        base_point : array-like, shape=[..., dim]
+            Base point.
+            Optional, default: None.
 
         Returns
         -------
-        norm : array-like, shape=[n_samples,]
+        norm : array-like, shape=[...,]
+            Norm.
         """
         sq_norm = self.squared_norm(vector, base_point)
         norm = gs.sqrt(sq_norm)
@@ -193,14 +258,18 @@ class RiemannianMetric(Connection):
 
         Parameters
         ----------
-        point_a : array-like, shape=[n_samples, dim]
-        point_b : array-like, shape=[n_samples, dim]
+        point_a : array-like, shape=[..., dim]
+            Point.
+        point_b : array-like, shape=[..., dim]
+            Point.
 
         Returns
         -------
-        sq_dist : array-like, shape=[n_samples,]
+        sq_dist : array-like, shape=[...,]
+            Squared distance.
         """
         log = self.log(point=point_b, base_point=point_a)
+
         sq_dist = self.squared_norm(vector=log, base_point=point_a)
         return sq_dist
 
@@ -212,12 +281,15 @@ class RiemannianMetric(Connection):
 
         Parameters
         ----------
-        point_a : array-like, shape=[n_samples, dim]
-        point_b : array-like, shape=[n_samples, dim]
+        point_a : array-like, shape=[..., dim]
+            Point.
+        point_b : array-like, shape=[..., dim]
+            Point.
 
         Returns
         -------
-        dist : array-like, shape=[n_samples,]
+        dist : array-like, shape=[...,]
+            Distance.
         """
         sq_dist = self.squared_dist(point_a, point_b)
         dist = gs.sqrt(sq_dist)
@@ -231,12 +303,13 @@ class RiemannianMetric(Connection):
 
         Parameters
         ----------
-        points
+        points : array-like, shape=[..., dim]
+            Points.
 
         Returns
         -------
-        diameter
-
+        diameter : float
+            Distance between two farthest points.
         """
         diameter = 0.0
         n_points = points.shape[0]
@@ -253,12 +326,15 @@ class RiemannianMetric(Connection):
 
         Parameters
         ----------
-        point
-        neighbors
+        point : array-like, shape=[..., dim]
+            Point.
+        neighbors : array-like, shape=[..., dim]
+            Neighbors.
+
         Returns
         -------
-        closest_neighbor_index
-
+        closest_neighbor_index : int
+            Index of closest neighbor.
         """
         dist = self.dist(point, neighbors)
         closest_neighbor_index = gs.argmin(dist)
