@@ -9,20 +9,19 @@ from geomstats.geometry.lie_algebra import MatrixLieAlgebra
 from geomstats.geometry.matrices import Matrices
 
 
-TOLERANCE = 1e-12
+TOLERANCE = 1e-8
 
 
 class SkewSymmetricMatrices(MatrixLieAlgebra):
-    """Class for skew-symmetric matrices."""
+    """Class for skew-symmetric matrices.
+
+    Parameters
+    ----------
+    n : int
+        Number of rows and columns.
+    """
 
     def __init__(self, n):
-        """Instantiate the class.
-
-        Parameters
-        ----------
-        n: int
-            The number of rows and columns.
-        """
         dimension = int(n * (n - 1) / 2)
         super(SkewSymmetricMatrices, self).__init__(dimension, n)
 
@@ -40,14 +39,16 @@ class SkewSymmetricMatrices(MatrixLieAlgebra):
 
         Parameters
         ----------
-        mat : array-like, shape=[n_samples, n, n]
-            The square matrix to check.
+        mat : array-like, shape=[..., n, n]
+            Square matrix to check.
         atol : float
             Tolerance for the equality evaluation.
+            Optional, default: TOLERANCE.
 
         Returns
         -------
-        belongs : bool
+        belongs : array-like, shape=[...,]
+            Boolean evaluating if matrix is skew symmetric.
         """
         return Matrices(self.n, self.n).is_skew_symmetric(mat=mat, atol=atol)
 
@@ -59,11 +60,13 @@ class SkewSymmetricMatrices(MatrixLieAlgebra):
 
         Parameters
         ----------
-        matrix_representation: array-like, shape=[n_samples, n, n]
+        matrix_representation : array-like, shape=[..., n, n]
+            Matrix.
 
         Returns
         -------
-        basis_representation: array-like, shape=[n_samples, dim]
+        basis_representation : array-like, shape=[..., dim]
+            Representation in the basis.
         """
         old_shape = gs.shape(matrix_representation)
         as_vector = gs.reshape(matrix_representation, (old_shape[0], -1))
