@@ -15,7 +15,13 @@ TOLERANCE = 1e-12
 
 
 class SPDMatrices(SymmetricMatrices, EmbeddedManifold):
-    """Class for the manifold of symmetric positive definite (SPD) matrices."""
+    """Class for the manifold of symmetric positive definite (SPD) matrices.
+
+    Parameters
+    ----------
+    n : int
+        Integer representing the shape of the matrices: n x n.
+    """
 
     def __init__(self, n):
         super(SPDMatrices, self).__init__(
@@ -23,17 +29,42 @@ class SPDMatrices(SymmetricMatrices, EmbeddedManifold):
             dim=int(n * (n + 1) / 2),
             embedding_manifold=GeneralLinear(n=n))
 
-    @staticmethod
-    def belongs(mat, atol=TOLERANCE):
-        """Check if a matrix is symmetric and invertible."""
-        is_symmetric = GeneralLinear.is_symmetric(mat)
+    def belongs(self, mat, atol=TOLERANCE):
+        """Check if a matrix is symmetric and invertible.
+
+        Parameters
+        ----------
+        mat : array-like, shape=[..., n, n]
+            Matrix to be checked.
+        atol : float
+            Tolerance.
+            Optional, default: TOLERANCE.
+
+        Returns
+        -------
+        belongs : array-like, shape=[...,]
+            Boolean denoting if mat is an SPD matrix.
+        """
+        is_symmetric = super(SPDMatrices, self).belongs(mat, atol)
         eigvalues, _ = gs.linalg.eigh(mat)
         is_positive = gs.all(eigvalues > 0, axis=-1)
         belongs = gs.logical_and(is_symmetric, is_positive)
         return belongs
 
     def random_uniform(self, n_samples=1):
-        """Define a log-uniform random sample of SPD matrices."""
+        """Sample in SPD(n) from the log-uniform distribution.
+
+        Parameters
+        ----------
+        n_samples : int
+            Number of samples.
+            Optional, default: 1.
+
+        Returns
+        -------
+        samples : array-like, shape=[..., n, n]
+            Points sampled in SPD(n).
+        """
         n = self.n
         size = (n_samples, n, n) if n_samples != 1 else (n, n)
 
@@ -43,7 +74,22 @@ class SPDMatrices(SymmetricMatrices, EmbeddedManifold):
         return spd_mat
 
     def random_tangent_vec_uniform(self, n_samples=1, base_point=None):
-        """Define a uniform random sample of tangent vectors."""
+        """Sample on the tangent space of SPD(n) from the uniform distribution.
+
+        Parameters
+        ----------
+        n_samples : int
+            Number of samples.
+            Optional, default: 1.
+        base_point : array-like, shape=[..., n, n]
+            Base point of the tangent space.
+            Optional, default: None.
+
+        Returns
+        -------
+        samples : array-like, shape=[..., n, n]
+            Points sampled in the tangent space at base_point.
+        """
         n = self.n
         size = (n_samples, n, n) if n_samples != 1 else (n, n)
 
@@ -74,18 +120,18 @@ class SPDMatrices(SymmetricMatrices, EmbeddedManifold):
         ----------
         power : float
             Power function to differentiate.
-        tangent_vec : array_like, shape=[n_samples, n, n]
-            Tangent vectors.
-        base_point : array_like, shape=[n_samples, n, n]
-            Base points.
+        tangent_vec : array_like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array_like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        eigvectors : array-like, shape=[n_samples, n, n]
-        transp_eigvectors : array-like, shape=[n_samples, n, n]
-        numerator : array-like, shape=[n_samples, n, n]
-        denominator : array-like, shape=[n_samples, n, n]
-        temp_result : array-like, shape=[n_samples, n, n]
+        eigvectors : array-like, shape=[..., n, n]
+        transp_eigvectors : array-like, shape=[..., n, n]
+        numerator : array-like, shape=[..., n, n]
+        denominator : array-like, shape=[..., n, n]
+        temp_result : array-like, shape=[..., n, n]
         """
         n_tangent_vecs, _, _ = tangent_vec.shape
         n_base_points, _, n = base_point.shape
@@ -147,23 +193,24 @@ class SPDMatrices(SymmetricMatrices, EmbeddedManifold):
     @classmethod
     @geomstats.vectorization.decorator(['else', 'else', 'matrix', 'matrix'])
     def differential_power(cls, power, tangent_vec, base_point):
-        """Compute the differential of the matrix power function.
+        r"""Compute the differential of the matrix power function.
 
-        Computes the differential of the power function on SPD
-        matrices (A^p=exp(p log(A))) at base_point applied to
-        tangent_vec.
+        Compute the differential of the power function on SPD(n)
+        (:math: `A^p=\exp(p \log(A))`) at base_point applied to tangent_vec.
 
         Parameters
         ----------
         power : int
-        tangent_vec : array_like, shape=[n_samples, n, n]
-            Tangent vectors.
-        base_point : array_like, shape=[n_samples, n, n]
-            Base points.
+            Power.
+        tangent_vec : array_like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array_like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        differential_power : array-like, shape=[n_samples, n, n]
+        differential_power : array-like, shape=[..., n, n]
+            Differential of the power function.
         """
         eigvectors, transp_eigvectors, numerator, denominator, temp_result =\
             cls.aux_differential_power(power, tangent_vec, base_point)
@@ -176,23 +223,25 @@ class SPDMatrices(SymmetricMatrices, EmbeddedManifold):
     @classmethod
     @geomstats.vectorization.decorator(['else', 'else', 'matrix', 'matrix'])
     def inverse_differential_power(cls, power, tangent_vec, base_point):
-        """Compute the inverse of the differential of the matrix power.
+        r"""Compute the inverse of the differential of the matrix power.
 
-        Computes the inverse of the differential of the power
-        function on SPD matrices (A^p=exp(p log(A))) at base_point
+        Compute the inverse of the differential of the power
+        function on SPD matrices (:math: `A^p=exp(p log(A))`) at base_point
         applied to tangent_vec.
 
         Parameters
         ----------
         power : int
-        tangent_vec : array_like, shape=[n_samples, n, n]
-            Tangent vectors.
-        base_point : array_like, shape=[n_samples, n, n]
-            Base points.
+            Power.
+        tangent_vec : array_like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array_like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        inverse_differential_power : array-like, shape=[n_samples, n, n]
+        inverse_differential_power : array-like, shape=[..., n, n]
+            Inverse of the differential of the power function.
         """
         eigvectors, transp_eigvectors, numerator, denominator, temp_result =\
             cls.aux_differential_power(power, tangent_vec, base_point)
@@ -207,19 +256,20 @@ class SPDMatrices(SymmetricMatrices, EmbeddedManifold):
     def differential_log(cls, tangent_vec, base_point):
         """Compute the differential of the matrix logarithm.
 
-        Computes the differential of the matrix logarithm on SPD
+        Compute the differential of the matrix logarithm on SPD
         matrices at base_point applied to tangent_vec.
 
         Parameters
         ----------
-        tangent_vec : array_like, shape=[n_samples, n, n]
-            Tangent vectors.
-        base_point : array_like, shape=[n_samples, n, n]
-            Base points.
+        tangent_vec : array_like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array_like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        differential_log : array-like, shape=[n_samples, n, n]
+        differential_log : array-like, shape=[..., n, n]
+            Differential of the matrix logarithm.
         """
         eigvectors, transp_eigvectors, numerator, denominator, temp_result =\
             cls.aux_differential_power(0, tangent_vec, base_point)
@@ -234,20 +284,21 @@ class SPDMatrices(SymmetricMatrices, EmbeddedManifold):
     def inverse_differential_log(cls, tangent_vec, base_point):
         """Compute the inverse of the differential of the matrix logarithm.
 
-        Computes the inverse of the differential of the matrix
+        Compute the inverse of the differential of the matrix
         logarithm on SPD matrices at base_point applied to
         tangent_vec.
 
         Parameters
         ----------
-        tangent_vec : array_like, shape=[n_samples, n, n]
-            Tangent vectors.
-        base_point : array_like, shape=[n_samples, n, n]
-            Base points.
+        tangent_vec : array_like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array_like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        inverse_differential_log : array-like, shape=[n_samples, n, n]
+        inverse_differential_log : array-like, shape=[..., n, n]
+            Inverse of the differential of the matrix logarithm.
         """
         eigvectors, transp_eigvectors, numerator, denominator, temp_result =\
             cls.aux_differential_power(0, tangent_vec, base_point)
@@ -267,14 +318,15 @@ class SPDMatrices(SymmetricMatrices, EmbeddedManifold):
 
         Parameters
         ----------
-        tangent_vec : array_like, shape=[n_samples, n, n]
-            Tangent vectors.
-        base_point : array_like, shape=[n_samples, n, n]
-            Base points.
+        tangent_vec : array_like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array_like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        differential_exp : array-like, shape=[n_samples, n, n]
+        differential_exp : array-like, shape=[..., n, n]
+            Differential of the matrix exponential.
         """
         eigvectors, transp_eigvectors, numerator, denominator, temp_result = \
             cls.aux_differential_power(math.inf, tangent_vec, base_point)
@@ -295,14 +347,15 @@ class SPDMatrices(SymmetricMatrices, EmbeddedManifold):
 
         Parameters
         ----------
-        tangent_vec : array_like, shape=[n_samples, n, n]
-            Tangent vectors.
-        base_point : array_like, shape=[n_samples, n, n]
-            Base points.
+        tangent_vec : array_like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array_like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        inverse_differential_exp : array-like, shape=[n_samples, n, n]
+        inverse_differential_exp : array-like, shape=[..., n, n]
+            Inverse of the differential of the matrix exponential.
         """
         eigvectors, transp_eigvectors, numerator, denominator, temp_result = \
             cls.aux_differential_power(math.inf, tangent_vec, base_point)
@@ -313,21 +366,21 @@ class SPDMatrices(SymmetricMatrices, EmbeddedManifold):
         return result
 
     @classmethod
-    def logm(cls, x):
+    def logm(cls, mat):
         """
         Compute the matrix log for a symmetric matrix.
 
         Parameters
         ----------
-        x : array_like, shape=[n_samples, n, n]
+        mat : array_like, shape=[..., n, n]
             Symmetric matrix.
 
         Returns
         -------
-        log : array_like, shape=[n_samples, n, n]
-            Logarithm of x.
+        log : array_like, shape=[..., n, n]
+            Matrix logarithm of mat.
         """
-        return cls.apply_func_to_eigvals(x, gs.log, check_positive=True)
+        return cls.apply_func_to_eigvals(mat, gs.log, check_positive=True)
 
 
 class SPDMetricAffine(RiemannianMetric):
@@ -336,14 +389,13 @@ class SPDMetricAffine(RiemannianMetric):
     def __init__(self, n, power_affine=1):
         """Build the affine-invariant metric.
 
-        Based on [TP2019]_.
-
         Parameters
         ----------
         n : int
-            Matrix dimension.
-        power_affine : int, optional
+            Integer representing the shape of the matrices: n x n.
+        power_affine : int
             Power transformation of the classical SPD metric.
+            Optional, default: 1.
 
         References
         ----------
@@ -354,24 +406,25 @@ class SPDMetricAffine(RiemannianMetric):
         dim = int(n * (n + 1) / 2)
         super(SPDMetricAffine, self).__init__(
             dim=dim,
-            signature=(dim, 0, 0))
+            signature=(dim, 0, 0),
+            default_point_type='matrix')
         self.n = n
         self.space = SPDMatrices(n)
         self.power_affine = power_affine
 
     @staticmethod
     def _aux_inner_product(tangent_vec_a, tangent_vec_b, inv_base_point):
-        """Compute the inner product (auxiliary).
+        """Compute the inner-product (auxiliary).
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[n_samples, n, n]
-        tangent_vec_b : array-like, shape=[n_samples, n, n]
-        inv_base_point : array-like, shape=[n_samples, n, n]
+        tangent_vec_a : array-like, shape=[..., n, n]
+        tangent_vec_b : array-like, shape=[..., n, n]
+        inv_base_point : array-like, shape=[..., n, n]
 
         Returns
         -------
-        inner_product : array-like, shape=[n_samples, n, n]
+        inner_product : array-like, shape=[..., n, n]
         """
         aux_a = gs.einsum(
             '...ij,...jk->...ik', inv_base_point, tangent_vec_a)
@@ -383,26 +436,30 @@ class SPDMetricAffine(RiemannianMetric):
         return inner_product
 
     def inner_product(self, tangent_vec_a, tangent_vec_b, base_point):
-        """Compute the affine-invariant inner product.
+        """Compute the affine-invariant inner-product.
 
-        Compute the inner product of tangent_vec_a and tangent_vec_b
+        Compute the inner-product of tangent_vec_a and tangent_vec_b
         at point base_point using the affine invariant Riemannian metric.
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[n_samples, n, n]
-        tangent_vec_b : array-like, shape=[n_samples, n, n]
-        base_point : array-like, shape=[n_samples, n, n]
+        tangent_vec_a : array-like, shape=[..., n, n]
+            Tangent vector at base point.
+        tangent_vec_b : array-like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array-like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        inner_product : array-like, shape=[n_samples, n, n]
+        inner_product : array-like, shape=[..., n, n]
+            Inner-product.
         """
         power_affine = self.power_affine
         spd_space = self.space
 
         if power_affine == 1:
-            inv_base_point = GeneralLinear.inv(base_point)
+            inv_base_point = GeneralLinear.inverse(base_point)
             inner_product = self._aux_inner_product(
                 tangent_vec_a, tangent_vec_b, inv_base_point)
         else:
@@ -427,19 +484,19 @@ class SPDMetricAffine(RiemannianMetric):
 
         Parameters
         ----------
-        tangent_vec : array-like, shape=[n_samples, n, n]
-        sqrt_base_point
-        inv_sqrt_base_point
+        tangent_vec : array-like, shape=[..., n, n]
+        sqrt_base_point : array-like, shape=[..., n, n]
+        inv_sqrt_base_point : array-like, shape=[..., n, n]
 
         Returns
         -------
-        exp
+        exp : array-like, shape=[..., n, n]
         """
         tangent_vec_at_id = gs.einsum(
             '...ij,...jk->...ik', inv_sqrt_base_point, tangent_vec)
         tangent_vec_at_id = gs.einsum(
             '...ij,...jk->...ik', tangent_vec_at_id, inv_sqrt_base_point)
-        tangent_vec_at_id = GeneralLinear.make_symmetric(tangent_vec_at_id)
+        tangent_vec_at_id = GeneralLinear.to_symmetric(tangent_vec_at_id)
         exp_from_id = SymmetricMatrices.expm(tangent_vec_at_id)
 
         exp = gs.einsum(
@@ -457,12 +514,15 @@ class SPDMetricAffine(RiemannianMetric):
 
         Parameters
         ----------
-        tangent_vec : array-like, shape=[n_samples, n, n]
-        base_point : array-like, shape=[n_samples, n, n]
+        tangent_vec : array-like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array-like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        exp : array-like, shape=[n_samples, n, n]
+        exp : array-like, shape=[..., n, n]
+            Riemannian exponential.
         """
         power_affine = self.power_affine
 
@@ -476,7 +536,7 @@ class SPDMetricAffine(RiemannianMetric):
                 power_affine, tangent_vec, base_point)
             power_sqrt_base_point = SymmetricMatrices.powerm(
                 base_point, power_affine / 2)
-            power_inv_sqrt_base_point = GeneralLinear.inv(
+            power_inv_sqrt_base_point = GeneralLinear.inverse(
                 power_sqrt_base_point)
             exp = self._aux_exp(
                 modified_tangent_vec,
@@ -492,19 +552,19 @@ class SPDMetricAffine(RiemannianMetric):
 
         Parameters
         ----------
-        point
-        sqrt_base_point
-        inv_sqrt_base_point
+        point : array-like, shape=[..., n, n]
+        sqrt_base_point : array-like, shape=[..., n, n]
+        inv_sqrt_base_point : array-like, shape=[.., n, n]
 
         Returns
         -------
-        log
+        log : array-like, shape=[..., n, n]
         """
         point_near_id = gs.einsum(
             '...ij,...jk->...ik', inv_sqrt_base_point, point)
         point_near_id = gs.einsum(
             '...ij,...jk->...ik', point_near_id, inv_sqrt_base_point)
-        point_near_id = GeneralLinear.make_symmetric(point_near_id)
+        point_near_id = GeneralLinear.to_symmetric(point_near_id)
         log_at_id = SPDMatrices.logm(point_near_id)
 
         log = gs.einsum(
@@ -522,12 +582,15 @@ class SPDMetricAffine(RiemannianMetric):
 
         Parameters
         ----------
-        point : array-like, shape=[n_samples, n, n]
-        base_point : array-like, shape=[n_samples, n, n]
+        point : array-like, shape=[..., n, n]
+            Point.
+        base_point : array-like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        log : array-like, shape=[n_samples, n, n]
+        log : array-like, shape=[..., n, n]
+            Riemannian logarithm of point at base_point.
         """
         power_affine = self.power_affine
 
@@ -553,12 +616,16 @@ class SPDMetricAffine(RiemannianMetric):
 
         Parameters
         ----------
-        initial_point
-        initial_tangent_vec
+        initial_point : array-like, shape=[..., n, n]
+            Initial point of the geodesic.
+        initial_tangent_vec : array-like, shape=[..., n, n]
+            Tangent vector at the initial point, the initial speed
+            of the geodesic.
 
         Returns
         -------
-        geodesic
+        geodesic : callable
+            Time-parameterized geodesic curve.
         """
         return super(SPDMetricAffine, self).geodesic(
             initial_point=initial_point,
@@ -580,21 +647,21 @@ class SPDMetricAffine(RiemannianMetric):
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[n_samples, dim + 1]
+        tangent_vec_a : array-like, shape=[..., dim + 1]
             Tangent vector at base point to be transported.
-        tangent_vec_b : array-like, shape=[n_samples, dim + 1]
+        tangent_vec_b : array-like, shape=[..., dim + 1]
             Tangent vector at base point, initial speed of the geodesic along
             which the parallel transport is computed.
-        base_point : array-like, shape=[n_samples, dim + 1]
-            point on the manifold of SPD matrices
+        base_point : array-like, shape=[..., dim + 1]
+            Point on the manifold of SPD matrices.
 
         Returns
         -------
-        transported_tangent_vec: array-like, shape=[n_samples, dim + 1]
+        transported_tangent_vec: array-like, shape=[..., dim + 1]
             Transported tangent vector at exp_(base_point)(tangent_vec_b).
         """
         end_point = self.exp(tangent_vec_b, base_point)
-        inverse_base_point = GeneralLinear.inv(base_point)
+        inverse_base_point = GeneralLinear.inverse(base_point)
         congruence_mat = GeneralLinear.mul(end_point, inverse_base_point)
         congruence_mat = gs.linalg.sqrtm(congruence_mat)
         return GeneralLinear.congruent(tangent_vec_a, congruence_mat)
@@ -603,7 +670,10 @@ class SPDMetricAffine(RiemannianMetric):
 class SPDMetricProcrustes(RiemannianMetric):
     """Class for the Procrustes metric on the SPD manifold.
 
-    Based on [BJL2017].
+    Parameters
+    ----------
+    n : int
+        Integer representing the shape of the matrices: n x n.
 
     References
     ----------
@@ -621,20 +691,24 @@ class SPDMetricProcrustes(RiemannianMetric):
         self.space = SPDMatrices(n)
 
     def inner_product(self, tangent_vec_a, tangent_vec_b, base_point):
-        """Compute the Procrustes inner product.
+        """Compute the Procrustes inner-product.
 
-        Compute the inner product of tangent_vec_a and tangent_vec_b
+        Compute the inner-product of tangent_vec_a and tangent_vec_b
         at point base_point using the Procrustes Riemannian metric.
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[n_samples, n, n]
-        tangent_vec_b : array-like, shape=[n_samples, n, n]
-        base_point : array-like, shape=[n_samples, n, n]
+        tangent_vec_a : array-like, shape=[..., n, n]
+            Tangent vector at base point.
+        tangent_vec_b : array-like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array-like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        inner_product : float
+        inner_product : array-like, shape=[...,]
+            Inner-product.
         """
         spd_space = self.space
         modified_tangent_vec_a =\
@@ -658,20 +732,24 @@ class SPDMetricEuclidean(RiemannianMetric):
         self.power_euclidean = power_euclidean
 
     def inner_product(self, tangent_vec_a, tangent_vec_b, base_point):
-        """Compute the Euclidean inner product.
+        """Compute the Euclidean inner-product.
 
-        Compute the inner product of tangent_vec_a and tangent_vec_b
+        Compute the inner-product of tangent_vec_a and tangent_vec_b
         at point base_point using the power-Euclidean metric.
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[n_samples, n, n]
-        tangent_vec_b : array-like, shape=[n_samples, n, n]
-        base_point : array-like, shape=[n_samples, n, n]
+        tangent_vec_a : array-like, shape=[..., n, n]
+            Tangent vector at base point.
+        tangent_vec_b : array-like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array-like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        inner_product : float
+        inner_product : array-like, shape=[...,]
+            Inner-product.
         """
         power_euclidean = self.power_euclidean
 
@@ -704,12 +782,15 @@ class SPDMetricEuclidean(RiemannianMetric):
 
         Parameters
         ----------
-        tangent_vec : array-like, shape=[n_samples, n, n]
-        base_point : array-like, shape=[n_samples, n, n]
+        tangent_vec : array-like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array-like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        exp_domain : array-like, shape=[n_samples, 2]
+        exp_domain : array-like, shape=[..., 2]
+            Interval of time where the geodesic is defined.
         """
         invsqrt_base_point = gs.linalg.powerm(base_point, -.5)
 
@@ -730,7 +811,13 @@ class SPDMetricEuclidean(RiemannianMetric):
 
 
 class SPDMetricLogEuclidean(RiemannianMetric):
-    """Class for the Log-Euclidean metric on the SPD manifold."""
+    """Class for the Log-Euclidean metric on the SPD manifold.
+
+    Parameters
+    ----------
+    n : int
+        Integer representing the shape of the matrices: n x n.
+    """
 
     def __init__(self, n):
         dim = int(n * (n + 1) / 2)
@@ -741,20 +828,24 @@ class SPDMetricLogEuclidean(RiemannianMetric):
         self.space = SPDMatrices(n)
 
     def inner_product(self, tangent_vec_a, tangent_vec_b, base_point):
-        """Compute the Log-Euclidean inner product.
+        """Compute the Log-Euclidean inner-product.
 
-        Compute the inner product of tangent_vec_a and tangent_vec_b
+        Compute the inner-product of tangent_vec_a and tangent_vec_b
         at point base_point using the log-Euclidean metric.
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[n_samples, n, n]
-        tangent_vec_b : array-like, shape=[n_samples, n, n]
-        base_point : array-like, shape=[n_samples, n, n]
+        tangent_vec_a : array-like, shape=[..., n, n]
+            Tangent vector at base point.
+        tangent_vec_b : array-like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array-like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        inner_product : float
+        inner_product : array-like, shape=[...,]
+            Inner-product.
         """
         spd_space = self.space
 
@@ -778,12 +869,15 @@ class SPDMetricLogEuclidean(RiemannianMetric):
 
         Parameters
         ----------
-        tangent_vec : array-like, shape=[n_samples, n, n]
-        base_point : array-like, shape=[n_samples, n, n]
+        tangent_vec : array-like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array-like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        exp : array-like, shape=[n_samples, n, n]
+        exp : array-like, shape=[..., n, n]
+            Riemannian exponential.
         """
         log_base_point = self.space.logm(base_point)
         dlog_tangent_vec = self.space.differential_log(tangent_vec, base_point)
@@ -800,12 +894,15 @@ class SPDMetricLogEuclidean(RiemannianMetric):
 
         Parameters
         ----------
-        point : array-like, shape=[n_samples, n, n]
-        base_point : array-like, shape=[n_samples, n, n]
+        point : array-like, shape=[..., n, n]
+            Point.
+        base_point : array-like, shape=[..., n, n]
+            Base point.
 
         Returns
         -------
-        log : array-like, shape=[n_samples, n, n]
+        log : array-like, shape=[..., n, n]
+            Riemannian logarithm.
         """
         log_base_point = SPDMatrices.logm(base_point)
         log_point = SPDMatrices.logm(point)
@@ -819,13 +916,16 @@ class SPDMetricLogEuclidean(RiemannianMetric):
 
         Parameters
         ----------
-        initial_point : array-like, shape=[n_samples, n, n]
-        initial_tangent_vec : array-like, shape=[n_samples, n, n]
+        initial_point : array-like, shape=[..., n, n]
+            Initial point of the geodesic.
+        initial_tangent_vec : array-like, shape=[..., n, n]
+            Tangent vector at the initial point, the initial speed
+            of the geodesic.
 
         Returns
         -------
-        path : callable
-            The time parameterized geodesic.
+        geodesic : callable
+            Time-parameterized geodesic curve.
         """
         def path(t):
             return self.exp(t * initial_tangent_vec, initial_point)
