@@ -7,7 +7,7 @@ from sklearn.base import BaseEstimator, ClusterMixin
 import geomstats.backend as gs
 
 
-# TODO(nkoep): Move this into the OnlineKMeans class.
+# TODO (nkoep): Move this into the OnlineKMeans class.
 
 def online_kmeans(X, metric, n_clusters, n_repetitions=20,
                   tolerance=1e-5, max_iter=5e4):
@@ -28,7 +28,7 @@ def online_kmeans(X, metric, n_clusters, n_repetitions=20,
 
     Parameters
     ----------
-    X : array-like, shape=[n_samples, n_features]
+    X : array-like, shape=[..., n_features]
         Input data. It is treated sequentially by the algorithm, i.e.
         one datum is chosen randomly at each iteration.
     metric : object
@@ -193,6 +193,7 @@ class OnlineKMeans(BaseEstimator, ClusterMixin):
 
         Returns
         -------
-        labels : Index of the cluster each sample belongs to.
+        labels : int
+            Index of the cluster each sample belongs to.
         """
         return self.metric.closest_neighbor_index(point, self.cluster_centers_)
