@@ -5,7 +5,7 @@ from math import log
 
 from scipy import linalg
 from scipy.special import gammaln
-from sklearn.decomposition.base import _BasePCA
+from sklearn.decomposition._base import _BasePCA
 from sklearn.utils.extmath import stable_cumsum
 from sklearn.utils.extmath import svd_flip
 from sklearn.utils.validation import check_array
@@ -35,8 +35,8 @@ def _assess_dimension_(spectrum, rank, n_samples, n_features):
 
     Returns
     -------
-    ll : float,
-        The log-likelihood
+    ll : float
+        Log-likelihood.
 
     Notes
     -----
@@ -79,7 +79,7 @@ def _assess_dimension_(spectrum, rank, n_samples, n_features):
 
 
 def _infer_dimension_(spectrum, n_samples, n_features):
-    """Infers the dimension of a dataset of shape (n_samples, n_features).
+    """Infer the dimension of a dataset of shape (n_samples, n_features).
 
     The dataset is described by its spectrum `spectrum`.
     """
@@ -91,17 +91,25 @@ def _infer_dimension_(spectrum, n_samples, n_features):
 
 
 class TangentPCA(_BasePCA):
-    """Tangent Principal component analysis (tPCA).
+    r"""Tangent Principal component analysis (tPCA).
 
     Linear dimensionality reduction using
     Singular Value Decomposition of the
     Riemannian Log of the data at the tangent space
     of the Frechet mean.
+
+    Parameters
+    ----------
+    metric : RiemannianMetric
+        Riemannian metric.
+    n_components : int
+        Number of principal components.
+        Optional, default: None.
     """
 
     def __init__(self, metric, n_components=None, copy=True,
                  whiten=False, tol=0.0, iterated_power='auto',
-                 random_state=None, point_type='vector'):
+                 random_state=None):
         self.metric = metric
         self.n_components = n_components
         self.copy = copy
@@ -109,7 +117,7 @@ class TangentPCA(_BasePCA):
         self.tol = tol
         self.iterated_power = iterated_power
         self.random_state = random_state
-        self.point_type = point_type
+        self.point_type = metric.default_point_type
         self.base_point_fit = None
 
     def fit(self, X, y=None, base_point=None):
@@ -117,13 +125,13 @@ class TangentPCA(_BasePCA):
 
         Parameters
         ----------
-        X : array-like, shape=[n_samples, n_features]
+        X : array-like, shape=[..., n_features]
             Training data, where n_samples is the number of samples
             and n_features is the number of features.
         y : Ignored (Compliance with scikit-learn interface)
-        base_point : array-like, shape=[n_samples, n_features], optional
+        base_point : array-like, shape=[..., n_features], optional
             Point at which to perform the tangent PCA
-            Optional, default to Frechet mean if None
+            Optional, default to Frechet mean if None.
 
         Returns
         -------
@@ -138,17 +146,18 @@ class TangentPCA(_BasePCA):
 
         Parameters
         ----------
-        X : array-like, shape=[n_samples, n_features]
+        X : array-like, shape=[..., n_features]
             Training data, where n_samples is the number of samples
             and n_features is the number of features.
         y : Ignored (Compliance with scikit-learn interface)
-        base_point : array-like, shape=[n_samples, n_features]
+        base_point : array-like, shape=[..., n_features]
             Point at which to perform the tangent PCA
-            Optional, default to Frechet mean if None
+            Optional, default to Frechet mean if None.
 
         Returns
         -------
-        X_new : array-like, shape (n_samples, n_components)
+        X_new : array-like, shape=[..., n_components]
+            Projected data.
         """
         U, S, _ = self._fit(X, base_point=base_point)
 
@@ -162,19 +171,20 @@ class TangentPCA(_BasePCA):
 
         Parameters
         ----------
-        X : array-like, shape=[n_samples, n_features]
+        X : array-like, shape=[..., n_features]
             Data, where n_samples is the number of samples
             and n_features is the number of features.
         y : Ignored (Compliance with scikit-learn interface)
 
         Returns
         -------
-        X_new : array-like, shape=[n_samples, n_components]
+        X_new : array-like, shape=[..., n_components]
+            Projected data.
         """
         tangent_vecs = self.metric.log(X, base_point=self.base_point_fit)
         if self.point_type == 'matrix':
             if Matrices.is_symmetric(tangent_vecs).all():
-                X = SymmetricMatrices.vector_from_symmetric_matrix(
+                X = SymmetricMatrices.to_vector(
                     tangent_vecs)
             else:
                 X = gs.reshape(tangent_vecs, (len(X), - 1))
@@ -191,13 +201,14 @@ class TangentPCA(_BasePCA):
 
         Parameters
         ----------
-        X : array-like, shape=[n_samples, n_components]
+        X : array-like, shape=[..., n_components]
             New data, where n_samples is the number of samples
             and n_components is the number of components.
 
         Returns
         -------
-        X_original array-like, shape=[n_samples, n_features]
+        X_original : array-like, shape=[..., n_features]
+            Original data.
         """
         scores = self.mean_ + gs.matmul(
             X, self.components_)
@@ -205,7 +216,7 @@ class TangentPCA(_BasePCA):
             if Matrices.is_symmetric(self.base_point_fit).all():
                 scores = SymmetricMatrices(
                     self.base_point_fit.shape[-1]
-                ).symmetric_matrix_from_vector(scores)
+                ).from_vector(scores)
             else:
                 dim = self.base_point_fit.shape[-1]
                 scores = gs.reshape(scores, (len(scores), dim, dim))
@@ -216,19 +227,18 @@ class TangentPCA(_BasePCA):
 
         Parameters
         ----------
-        X : array-like, shape=[n_samples, n_features]
+        X : array-like, shape=[..., n_features]
             Training data, where n_samples is the number of samples
             and n_features is the number of features.
         y : Ignored (Compliance with scikit-learn interface)
-        base_point : array-like, shape=[n_samples, n_features]
-            Point at which to perform the tangent PCA
-            Optional, default to Frechet mean if None
-        point_type : str, {'vector', 'matrix'}
-            Optional
+        base_point : array-like, shape=[..., n_features]
+            Point at which to perform the tangent PCA.
+            Optional, default to Frechet mean if None.
 
         Returns
         -------
-        U, S, V: SVD decomposition
+        U, S, V : array-like
+            Matrices of the SVD decomposition
         """
         if base_point is None:
             mean = FrechetMean(metric=self.metric, point_type=self.point_type)
@@ -239,7 +249,7 @@ class TangentPCA(_BasePCA):
 
         if self.point_type == 'matrix':
             if Matrices.is_symmetric(tangent_vecs).all():
-                X = SymmetricMatrices.vector_from_symmetric_matrix(
+                X = SymmetricMatrices.to_vector(
                     tangent_vecs)
             else:
                 X = gs.reshape(tangent_vecs, (len(X), - 1))
