@@ -1,13 +1,11 @@
-"""
-Unit tests for the Euclidean space.
-"""
+"""Unit tests for the Euclidean space."""
 
 import geomstats.backend as gs
 import geomstats.tests
 from geomstats.geometry.euclidean import Euclidean
 
 
-class TestEuclideanMethods(geomstats.tests.TestCase):
+class TestEuclidean(geomstats.tests.TestCase):
     def setUp(self):
         gs.random.seed(1234)
 
