@@ -17,7 +17,7 @@ from geomstats.geometry.poincare_ball import PoincareBall
 RTOL = 1e-6
 
 
-class TestHyperbolicMethods(geomstats.tests.TestCase):
+class TestHyperbolic(geomstats.tests.TestCase):
     def setUp(self):
         gs.random.seed(1234)
         self.dimension = 3
@@ -42,7 +42,7 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         base_point = gs.array([1., 0., 0., 0.])
         self.assertTrue(self.space.belongs(base_point))
 
-        tangent_vec = self.space.projection_to_tangent_space(
+        tangent_vec = self.space.to_tangent(
             vector=gs.array([1., 2., 1., 3.]),
             base_point=base_point)
 
@@ -51,7 +51,7 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
 
         self.assertAllClose(result, expected)
 
-        result = self.space.projection_to_tangent_space(
+        result = self.space.to_tangent(
             vector=gs.array([1., 2., 1., 3.]),
             base_point=base_point)
         expected = tangent_vec
@@ -69,14 +69,12 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         point_ext = self.space.from_coordinates(point_int, 'intrinsic')
         result = self.space.to_coordinates(point_ext, 'intrinsic')
         expected = point_int
-        expected = helper.to_vector(expected)
         self.assertAllClose(result, expected)
 
         point_ext = gs.array([2.0, 1.0, 1.0, 1.0])
         point_int = self.space.to_coordinates(point_ext, 'intrinsic')
         result = self.space.from_coordinates(point_int, 'intrinsic')
         expected = point_ext
-        expected = helper.to_vector(expected)
 
         self.assertAllClose(result, expected)
 
@@ -184,14 +182,13 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         base_point = gs.array([1., 0., 0.])
         self.assertTrue(H2.belongs(base_point))
 
-        tangent_vec = H2.projection_to_tangent_space(
+        tangent_vec = H2.to_tangent(
             vector=gs.array([1., 2., 1.]),
             base_point=base_point)
         exp = METRIC.exp(tangent_vec=tangent_vec,
                          base_point=base_point)
         self.assertTrue(H2.belongs(exp))
 
-    @geomstats.tests.np_and_pytorch_only
     def test_exp_vectorization(self):
         n_samples = 3
         dim = self.dimension + 1
@@ -206,42 +203,47 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
             [5.0, math.sqrt(8), math.sqrt(8), math.sqrt(8)],
             [1.0, 0.0, 0.0, 0.0]])
 
-        one_tangent_vec = self.space.projection_to_tangent_space(
+        one_tangent_vec = self.space.to_tangent(
             one_vec, base_point=one_base_point)
         result = self.metric.exp(one_tangent_vec, one_base_point)
         self.assertAllClose(gs.shape(result), (dim,))
 
-        n_tangent_vecs = self.space.projection_to_tangent_space(
+        n_tangent_vecs = self.space.to_tangent(
             n_vecs, base_point=one_base_point)
         result = self.metric.exp(n_tangent_vecs, one_base_point)
         self.assertAllClose(gs.shape(result), (n_samples, dim))
 
-        expected = gs.zeros((n_samples, dim))
+        expected = []
 
         for i in range(n_samples):
-            expected[i] = self.metric.exp(n_tangent_vecs[i], one_base_point)
+            expected.append(self.metric.exp(n_tangent_vecs[i], one_base_point))
+        expected = gs.stack(expected, axis=0)
         expected = helper.to_vector(gs.array(expected))
         self.assertAllClose(result, expected)
 
-        one_tangent_vec = self.space.projection_to_tangent_space(
+        one_tangent_vec = self.space.to_tangent(
             one_vec, base_point=n_base_points)
         result = self.metric.exp(one_tangent_vec, n_base_points)
         self.assertAllClose(gs.shape(result), (n_samples, dim))
 
-        expected = gs.zeros((n_samples, dim))
+        expected = []
         for i in range(n_samples):
-            expected[i] = self.metric.exp(one_tangent_vec[i], n_base_points[i])
+            expected.append(
+                self.metric.exp(one_tangent_vec[i], n_base_points[i]))
+        expected = gs.stack(expected, axis=0)
         expected = helper.to_vector(gs.array(expected))
         self.assertAllClose(result, expected)
 
-        n_tangent_vecs = self.space.projection_to_tangent_space(
+        n_tangent_vecs = self.space.to_tangent(
             n_vecs, base_point=n_base_points)
         result = self.metric.exp(n_tangent_vecs, n_base_points)
         self.assertAllClose(gs.shape(result), (n_samples, dim))
 
-        expected = gs.zeros((n_samples, dim))
+        expected = []
         for i in range(n_samples):
-            expected[i] = self.metric.exp(n_tangent_vecs[i], n_base_points[i])
+            expected.append(
+                self.metric.exp(n_tangent_vecs[i], n_base_points[i]))
+        expected = gs.stack(expected, axis=0)
         expected = helper.to_vector(gs.array(expected))
         self.assertAllClose(result, expected)
 
@@ -280,11 +282,11 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         base_point = gs.array(
             [1.16563816, 0.36381045, -0.47000603, 0.07381469])
 
-        tangent_vec_a = self.space.projection_to_tangent_space(
+        tangent_vec_a = self.space.to_tangent(
             vector=gs.array([10., 200., 1., 1.]),
             base_point=base_point)
 
-        tangent_vec_b = self.space.projection_to_tangent_space(
+        tangent_vec_b = self.space.to_tangent(
             vector=gs.array([11., 20., -21., 0.]),
             base_point=base_point)
 
@@ -346,7 +348,6 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_and_tf_only
     def test_exp_and_log_and_projection_to_tangent_space_general_case(self):
         """
         Test that the Riemannian exponential
@@ -358,7 +359,7 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         # General case
         base_point = gs.array([4.0, 1., 3.0, math.sqrt(5)])
         vector = gs.array([2.0, 1.0, 1.0, 1.0])
-        vector = self.space.projection_to_tangent_space(
+        vector = self.space.to_tangent(
             vector=vector,
             base_point=base_point)
         exp = self.metric.exp(tangent_vec=vector, base_point=base_point)
@@ -378,7 +379,7 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
     def test_exp_and_dist_and_projection_to_tangent_space(self):
         base_point = gs.array([4.0, 1., 3.0, math.sqrt(5)])
         vector = gs.array([0.001, 0., -.00001, -.00003])
-        tangent_vec = self.space.projection_to_tangent_space(
+        tangent_vec = self.space.to_tangent(
             vector=vector,
             base_point=base_point)
         exp = self.metric.exp(
@@ -392,12 +393,31 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         self.assertAllClose(result, expected, atol=1e-2)
 
     def test_geodesic_and_belongs(self):
-        # TODO(nina): Fix this tests, as it fails when geodesic goes "too far"
         initial_point = gs.array([4.0, 1., 3.0, math.sqrt(5)])
         n_geodesic_points = 100
         vector = gs.array([1., 0., 0., 0.])
 
-        initial_tangent_vec = self.space.projection_to_tangent_space(
+        initial_tangent_vec = self.space.to_tangent(
+            vector=vector,
+            base_point=initial_point)
+        geodesic = self.metric.geodesic(
+            initial_point=initial_point,
+            initial_tangent_vec=initial_tangent_vec)
+
+        t = gs.linspace(start=0., stop=1., num=n_geodesic_points)
+        points = geodesic(t)
+        result = self.space.belongs(points)
+        expected = n_geodesic_points * [True]
+
+        self.assertAllClose(result, expected)
+
+    @geomstats.tests.np_only
+    def test_geodesic_and_belongs_large_initial_velocity(self):
+        initial_point = gs.array([4.0, 1., 3.0, math.sqrt(5)])
+        n_geodesic_points = 100
+        vector = gs.array([3., 0., 0., 0.])
+
+        initial_tangent_vec = self.space.to_tangent(
             vector=vector,
             base_point=initial_point)
         geodesic = self.metric.geodesic(
@@ -425,23 +445,22 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
 
         exp = self.metric.exp(tangent_vec=vector, base_point=base_point)
         result = self.metric.log(point=exp, base_point=base_point)
-        expected = self.space.projection_to_tangent_space(
+        expected = self.space.to_tangent(
             vector=vector,
             base_point=base_point)
 
         self.assertAllClose(result, expected, atol=1e-8)
 
-    @geomstats.tests.np_only
     def test_scaled_inner_product(self):
-        base_point_intrinsic = gs.array([1, 1, 1])
+        base_point_intrinsic = gs.array([1., 1., 1.])
         base_point = self.space.from_coordinates(
-            base_point_intrinsic, "intrinsic")
-        tangent_vec_a = gs.array([1, 2, 3, 4])
-        tangent_vec_b = gs.array([5, 6, 7, 8])
-        tangent_vec_a = self.space.projection_to_tangent_space(
+            base_point_intrinsic, 'intrinsic')
+        tangent_vec_a = gs.array([1., 2., 3., 4.])
+        tangent_vec_b = gs.array([5., 6., 7., 8.])
+        tangent_vec_a = self.space.to_tangent(
             tangent_vec_a,
             base_point)
-        tangent_vec_b = self.space.projection_to_tangent_space(
+        tangent_vec_b = self.space.to_tangent(
             tangent_vec_b,
             base_point)
         scale = 2
@@ -461,13 +480,12 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         expected = scale ** 2 * inner_product_default_metric
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_scaled_squared_norm(self):
-        base_point_intrinsic = gs.array([1, 1, 1])
-        base_point = self.space.from_coordinates(base_point_intrinsic,
-                                                 'intrinsic')
-        tangent_vec = gs.array([1, 2, 3, 4])
-        tangent_vec = self.space.projection_to_tangent_space(
+        base_point_intrinsic = gs.array([1., 1., 1.])
+        base_point = self.space.from_coordinates(
+            base_point_intrinsic, 'intrinsic')
+        tangent_vec = gs.array([1., 2., 3., 4.])
+        tangent_vec = self.space.to_tangent(
             tangent_vec, base_point)
         scale = 2
         default_space = Hyperboloid(dim=self.dimension)
@@ -480,10 +498,9 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         expected = scale ** 2 * squared_norm_default_metric
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_scaled_distance(self):
-        point_a_intrinsic = gs.array([1, 2, 3])
-        point_b_intrinsic = gs.array([4, 5, 6])
+        point_a_intrinsic = gs.array([1., 2., 3.])
+        point_b_intrinsic = gs.array([4., 5., 6.])
         point_a = self.space.from_coordinates(point_a_intrinsic, 'intrinsic')
         point_b = self.space.from_coordinates(point_b_intrinsic, 'intrinsic')
         scale = 2
