@@ -11,7 +11,7 @@ KAPPA_ESTIMATION_TOL = 1e-2
 ONLINE_KMEANS_TOL = 2e-2
 
 
-class TestHypersphereMethods(geomstats.tests.TestCase):
+class TestHypersphere(geomstats.tests.TestCase):
     def setUp(self):
         gs.random.seed(1234)
 
@@ -20,9 +20,9 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         self.metric = self.space.metric
         self.n_samples = 10
 
-    @geomstats.tests.np_and_pytorch_only
     def test_random_uniform_and_belongs(self):
-        """
+        """Test random uniform and belongs.
+
         Test that the random uniform method samples
         on the hypersphere space.
         """
@@ -33,12 +33,19 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
 
         self.assertAllClose(expected, result)
 
-    @geomstats.tests.np_and_pytorch_only
     def test_random_uniform(self):
         point = self.space.random_uniform()
 
         self.assertAllClose(gs.shape(point), (self.dimension + 1,))
 
+    def test_replace_values(self):
+        points = gs.ones((3, 5))
+        new_points = gs.zeros((2, 5))
+        indcs = [True, False, True]
+        update = self.space._replace_values(points, new_points, indcs)
+        self.assertAllClose(update, gs.stack(
+            [gs.zeros(5), gs.ones(5), gs.zeros(5)]))
+
     def test_projection_and_belongs(self):
         point = gs.array([1., 2., 3., 4., 5.])
         proj = self.space.projection(point)
@@ -70,7 +77,8 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         self.assertAllClose(result, expected)
 
     def test_intrinsic_and_extrinsic_coords_vectorization(self):
-        """
+        """Test change of coordinates.
+
         Test that the composition of
         intrinsic_to_extrinsic_coords and
         extrinsic_to_intrinsic_coords
@@ -95,9 +103,9 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_and_pytorch_only
     def test_log_and_exp_general_case(self):
-        """
+        """Test Log and Exp.
+
         Test that the Riemannian exponential
         and the Riemannian logarithm are inverse.
 
@@ -119,9 +127,9 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
 
         self.assertAllClose(result, expected, atol=1e-6)
 
-    @geomstats.tests.np_and_pytorch_only
     def test_log_and_exp_edge_case(self):
-        """
+        """Test Log and Exp.
+
         Test that the Riemannian exponential
         and the Riemannian logarithm are inverse.
 
@@ -145,13 +153,12 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_and_pytorch_only
     def test_exp_vectorization_single_samples(self):
         dim = self.dimension + 1
 
         one_vec = self.space.random_uniform()
         one_base_point = self.space.random_uniform()
-        one_tangent_vec = self.space.projection_to_tangent_space(
+        one_tangent_vec = self.space.to_tangent(
             one_vec, base_point=one_base_point)
 
         result = self.metric.exp(one_tangent_vec, one_base_point)
@@ -169,7 +176,6 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         result = self.metric.exp(one_tangent_vec, one_base_point)
         self.assertAllClose(gs.shape(result), (1, dim))
 
-    @geomstats.tests.np_and_pytorch_only
     def test_exp_vectorization_n_samples(self):
         n_samples = self.n_samples
         dim = self.dimension + 1
@@ -179,25 +185,24 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         n_vecs = self.space.random_uniform(n_samples=n_samples)
         n_base_points = self.space.random_uniform(n_samples=n_samples)
 
-        n_tangent_vecs = self.space.projection_to_tangent_space(
+        n_tangent_vecs = self.space.to_tangent(
             n_vecs, base_point=one_base_point)
         result = self.metric.exp(n_tangent_vecs, one_base_point)
 
         self.assertAllClose(gs.shape(result), (n_samples, dim))
 
-        one_tangent_vec = self.space.projection_to_tangent_space(
+        one_tangent_vec = self.space.to_tangent(
             one_vec, base_point=n_base_points)
         result = self.metric.exp(one_tangent_vec, n_base_points)
 
         self.assertAllClose(gs.shape(result), (n_samples, dim))
 
-        n_tangent_vecs = self.space.projection_to_tangent_space(
+        n_tangent_vecs = self.space.to_tangent(
             n_vecs, base_point=n_base_points)
         result = self.metric.exp(n_tangent_vecs, n_base_points)
 
         self.assertAllClose(gs.shape(result), (n_samples, dim))
 
-    @geomstats.tests.np_and_pytorch_only
     def test_log_vectorization_single_samples(self):
         dim = self.dimension + 1
 
@@ -219,7 +224,6 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         result = self.metric.log(one_point, one_base_point)
         self.assertAllClose(gs.shape(result), (1, dim))
 
-    @geomstats.tests.np_and_pytorch_only
     def test_log_vectorization_n_samples(self):
         n_samples = self.n_samples
         dim = self.dimension + 1
@@ -241,9 +245,9 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         result = self.metric.log(n_points, n_base_points)
         self.assertAllClose(gs.shape(result), (n_samples, dim))
 
-    @geomstats.tests.np_and_pytorch_only
     def test_exp_and_log_and_projection_to_tangent_space_general_case(self):
-        """
+        """Test Log and Exp.
+
         Test that the Riemannian exponential
         and the Riemannian logarithm are inverse.
 
@@ -252,28 +256,34 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         NB: points on the n-dimensional sphere are
         (n+1)-D vectors of norm 1.
         """
-        # TODO(nina): Fix that this test fails, also in numpy
         # Riemannian Exp then Riemannian Log
         # General case
         # NB: Riemannian log gives a regularized tangent vector,
         # so we take the norm modulo 2 * pi.
         base_point = gs.array([0., -3., 0., 3., 4.])
         base_point = base_point / gs.linalg.norm(base_point)
-        vector = gs.array([9., 5., 0., 0., -1.])
-        vector = self.space.projection_to_tangent_space(
+
+        vector = gs.array([3., 2., 0., 0., -1.])
+        vector = self.space.to_tangent(
             vector=vector, base_point=base_point)
 
-        # exp = self.metric.exp(tangent_vec=vector, base_point=base_point)
-        # result = self.metric.log(point=exp, base_point=base_point)
+        exp = self.metric.exp(tangent_vec=vector, base_point=base_point)
+        result = self.metric.log(point=exp, base_point=base_point)
 
         expected = vector
         norm_expected = gs.linalg.norm(expected)
         regularized_norm_expected = gs.mod(norm_expected, 2 * gs.pi)
         expected = expected / norm_expected * regularized_norm_expected
 
-    @geomstats.tests.np_and_pytorch_only
+        # The Log can be the opposite vector on the tangent space,
+        # whose Exp gives the base_point
+        are_close = gs.allclose(result, expected)
+        norm_2pi = gs.isclose(gs.linalg.norm(result - expected), 2 * gs.pi)
+        self.assertTrue(are_close or norm_2pi)
+
     def test_exp_and_log_and_projection_to_tangent_space_edge_case(self):
-        """
+        """Test Log and Exp.
+
         Test that the Riemannian exponential
         and the Riemannian logarithm are inverse.
 
@@ -287,12 +297,12 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         base_point = gs.array([10., -2., -.5, 34., 3.])
         base_point = base_point / gs.linalg.norm(base_point)
         vector = 1e-10 * gs.array([.06, -51., 6., 5., 3.])
-        vector = self.space.projection_to_tangent_space(
+        vector = self.space.to_tangent(
             vector=vector, base_point=base_point)
 
         exp = self.metric.exp(tangent_vec=vector, base_point=base_point)
         result = self.metric.log(point=exp, base_point=base_point)
-        expected = self.space.projection_to_tangent_space(
+        expected = self.space.to_tangent(
             vector=vector, base_point=base_point)
 
         self.assertAllClose(result, expected, atol=1e-8)
@@ -312,7 +322,6 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_and_pytorch_only
     def test_squared_dist_vectorization_single_sample(self):
         one_point_a = self.space.random_uniform()
         one_point_b = self.space.random_uniform()
@@ -332,7 +341,6 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         result = self.metric.squared_dist(one_point_a, one_point_b)
         self.assertAllClose(gs.shape(result), (1,))
 
-    @geomstats.tests.np_and_pytorch_only
     def test_squared_dist_vectorization_n_samples(self):
         n_samples = self.n_samples
 
@@ -411,12 +419,11 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_and_pytorch_only
     def test_exp_and_dist_and_projection_to_tangent_space(self):
         base_point = gs.array([16., -2., -2.5, 84., 3.])
         base_point = base_point / gs.linalg.norm(base_point)
         vector = gs.array([9., 0., -1., -2., 1.])
-        tangent_vec = self.space.projection_to_tangent_space(
+        tangent_vec = self.space.to_tangent(
             vector=vector, base_point=base_point)
 
         exp = self.metric.exp(
@@ -425,7 +432,6 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         expected = gs.linalg.norm(tangent_vec) % (2 * gs.pi)
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_and_pytorch_only
     def test_exp_and_dist_and_projection_to_tangent_space_vec(self):
         base_point = gs.array([
             [16., -2., -2.5, 84., 3.],
@@ -439,7 +445,7 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
             [[9., 0., -1., -2., 1.],
              [9., 0., -1., -2., 1]])
 
-        tangent_vec = self.space.projection_to_tangent_space(
+        tangent_vec = self.space.to_tangent(
             vector=vector, base_point=base_point)
 
         exp = self.metric.exp(
@@ -450,23 +456,40 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_and_pytorch_only
     def test_geodesic_and_belongs(self):
-        n_geodesic_points = 100
-        initial_point = self.space.random_uniform()
-        vector = gs.array([2., 0., -1., -2., 1.])
-        initial_tangent_vec = self.space.projection_to_tangent_space(
+        n_geodesic_points = 10
+        initial_point = self.space.random_uniform(2)
+        vector = gs.array([[2., 0., -1., -2., 1.]] * 2)
+        initial_tangent_vec = self.space.to_tangent(
             vector=vector, base_point=initial_point)
         geodesic = self.metric.geodesic(
             initial_point=initial_point,
             initial_tangent_vec=initial_tangent_vec)
-
         t = gs.linspace(start=0., stop=1., num=n_geodesic_points)
         points = geodesic(t)
+        result = gs.stack([self.space.belongs(pt) for pt in points])
+        self.assertTrue(gs.all(result))
 
+        initial_point = initial_point[0]
+        initial_tangent_vec = initial_tangent_vec[0]
+        geodesic = self.metric.geodesic(
+            initial_point=initial_point,
+            initial_tangent_vec=initial_tangent_vec)
+        points = geodesic(t)
         result = self.space.belongs(points)
         expected = gs.array(n_geodesic_points * [True])
+        self.assertAllClose(expected, result)
 
+    def test_geodesic_end_point(self):
+        n_geodesic_points = 10
+        initial_point = self.space.random_uniform(4)
+        geodesic = self.metric.geodesic(
+            initial_point=initial_point[:2],
+            end_point=initial_point[2:])
+        t = gs.linspace(start=0., stop=1., num=n_geodesic_points)
+        points = geodesic(t)
+        result = points[-1]
+        expected = initial_point[2:]
         self.assertAllClose(expected, result)
 
     def test_inner_product(self):
@@ -562,11 +585,8 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         expected = gs.pi
         self.assertAllClose(expected, result)
 
-    @geomstats.tests.np_and_pytorch_only
     def test_closest_neighbor_index(self):
-        """
-        Check that the closest neighbor is one of neighbors.
-        """
+        """Check that the closest neighbor is one of neighbors."""
         n_samples = 10
         points = self.space.random_uniform(n_samples=n_samples)
         point = points[0, :]
@@ -578,7 +598,6 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         result = test > 0
         self.assertTrue(result)
 
-    @geomstats.tests.np_and_pytorch_only
     def test_sample_von_mises_fisher(self):
         """
         Check that the maximum likelihood estimates of the mean and
@@ -602,7 +621,7 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
             gs.allclose(result, expected, atol=MEAN_ESTIMATION_TOL)
         )
         # check concentration parameter for dispersed distribution
-        kappa = 1
+        kappa = 1.
         points = sphere.random_von_mises_fisher(kappa, n_points)
         sum_points = gs.sum(points, axis=0)
         mean_norm = gs.linalg.norm(sum_points) / n_points
@@ -618,12 +637,10 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
             denominator = 1. - ratio**2 - (p - 1.) * ratio / kappa_estimate
             mean_norm = gs.cast(mean_norm, gs.float64)
             kappa_estimate = kappa_estimate - (ratio - mean_norm) / denominator
-        expected = kappa
         result = kappa_estimate
-        self.assertTrue(
-            gs.allclose(result, expected, atol=KAPPA_ESTIMATION_TOL))
+        expected = kappa
+        self.assertAllClose(result, expected, atol=KAPPA_ESTIMATION_TOL)
 
-    @geomstats.tests.np_and_pytorch_only
     def test_spherical_to_extrinsic(self):
         """
         Check vectorization of conversion from spherical
@@ -637,7 +654,6 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         expected = gs.array([1., 0., 0.])
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_and_pytorch_only
     def test_spherical_to_extrinsic_vectorization(self):
         dim = 2
         sphere = Hypersphere(dim)
@@ -646,10 +662,9 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         result = sphere.spherical_to_extrinsic(points_spherical)
         expected = gs.array(
             [[1., 0., 0.],
-             [gs.sqrt(2) / 4, gs.sqrt(2) / 4, gs.sqrt(3) / 2]])
+             [gs.sqrt(2.) / 4., gs.sqrt(2.) / 4., gs.sqrt(3.) / 2.]])
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_and_pytorch_only
     def test_tangent_spherical_to_extrinsic(self):
         """
         Check vectorization of conversion from spherical
@@ -682,3 +697,53 @@ class TestHypersphereMethods(geomstats.tests.TestCase):
         result = christoffel.shape
         expected = gs.array([2, dim, dim, dim])
         self.assertAllClose(result, expected)
+
+    @geomstats.tests.np_and_tf_only
+    def test_parallel_transport_vectorization(self):
+        sphere = Hypersphere(2)
+        n_samples = 4
+
+        def is_isometry(tan_a, trans_a, endpoint):
+            is_tangent = gs.isclose(
+                sphere.metric.inner_product(endpoint, trans_a), 0., atol=1e-6)
+            is_equinormal = gs.isclose(
+                gs.linalg.norm(trans_a, axis=-1),
+                gs.linalg.norm(tan_a, axis=-1))
+            return gs.logical_and(is_tangent, is_equinormal)
+
+        base_point = sphere.random_uniform(n_samples)
+        tan_vec_a = sphere.to_tangent(gs.random.rand(n_samples, 3), base_point)
+        tan_vec_b = sphere.to_tangent(gs.random.rand(n_samples, 3), base_point)
+        end_point = sphere.metric.exp(tan_vec_b, base_point)
+
+        transported = sphere.metric.parallel_transport(
+            tan_vec_a, tan_vec_b, base_point)
+        result = is_isometry(tan_vec_a, transported, end_point)
+        self.assertTrue(gs.all(result))
+
+        base_point = base_point[0]
+        tan_vec_a = sphere.to_tangent(tan_vec_a, base_point)
+        tan_vec_b = sphere.to_tangent(tan_vec_b, base_point)
+        end_point = sphere.metric.exp(tan_vec_b, base_point)
+        transported = sphere.metric.parallel_transport(
+            tan_vec_a, tan_vec_b, base_point)
+        result = is_isometry(tan_vec_a, transported, end_point)
+        self.assertTrue(gs.all(result))
+
+        one_tan_vec_a = tan_vec_a[0]
+        transported = sphere.metric.parallel_transport(
+            one_tan_vec_a, tan_vec_b, base_point)
+        result = is_isometry(one_tan_vec_a, transported, end_point)
+        self.assertTrue(gs.all(result))
+
+        one_tan_vec_b = tan_vec_b[0]
+        end_point = end_point[0]
+        transported = sphere.metric.parallel_transport(
+            tan_vec_a, one_tan_vec_b, base_point)
+        result = is_isometry(tan_vec_a, transported, end_point)
+        self.assertTrue(gs.all(result))
+
+        transported = sphere.metric.parallel_transport(
+            one_tan_vec_a, one_tan_vec_b, base_point)
+        result = is_isometry(one_tan_vec_a, transported, end_point)
+        self.assertTrue(result)
