@@ -1,6 +1,4 @@
-"""
-Unit tests for landmarks space.
-"""
+"""Unit tests for landmarks space."""
 
 import geomstats.backend as gs
 import geomstats.tests
@@ -8,8 +6,8 @@ from geomstats.geometry.hypersphere import Hypersphere
 from geomstats.geometry.landmarks import Landmarks
 
 
-class TestLandmarksMethods(geomstats.tests.TestCase):
-    @geomstats.tests.np_and_pytorch_only
+class TestLandmarks(geomstats.tests.TestCase):
+
     def setUp(self):
         s2 = Hypersphere(dim=2)
         r3 = s2.embedding_manifold
@@ -40,36 +38,30 @@ class TestLandmarksMethods(geomstats.tests.TestCase):
         self.atol = 1e-6
         gs.random.seed(1234)
         self.space_landmarks_in_euclidean_3d = Landmarks(
-            ambient_manifold=r3)
+            ambient_manifold=r3, n_landmarks=self.n_sampling_points)
         self.space_landmarks_in_sphere_2d = Landmarks(
-            ambient_manifold=s2)
-        self.l2_metric_s2 = self.space_landmarks_in_sphere_2d.l2_metric
-        self.l2_metric_r3 = self.space_landmarks_in_euclidean_3d.l2_metric
+            ambient_manifold=s2, n_landmarks=self.n_sampling_points)
+        self.l2_metric_s2 = self.space_landmarks_in_sphere_2d.metric
+        self.l2_metric_r3 = self.space_landmarks_in_euclidean_3d.metric
         self.landmarks_a = landmark_set_a
         self.landmarks_b = landmark_set_b
         self.landmarks_c = landmark_set_c
 
-    @geomstats.tests.np_and_pytorch_only
     def test_belongs(self):
         result = self.space_landmarks_in_sphere_2d.belongs(self.landmarks_a)
         expected = True
         self.assertAllClose(result, expected)
 
-    # TODO(ninamiolane): Uncomment when belongs is vectorized
-    # @geomstats.tests.np_and_pytorch_only
-    # def test_belongs_vectorization(self):
-    #     landmark_sets = gs.array([self.landmarks_a, self.landmarks_b])
-    #     result = self.space_landmarks_in_sphere_2d.belongs(landmark_sets)
-    #     expected = gs.array([True, True])
-    #     self.assertAllClose(result, expected)
+    def test_belongs_vectorization(self):
+        landmark_sets = gs.array([self.landmarks_a, self.landmarks_b])
+        result = self.space_landmarks_in_sphere_2d.belongs(landmark_sets)
+        expected = gs.array([True, True])
+        self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_l2_metric_log_and_squared_norm_and_dist(self):
-        """
-        Test that squared norm of logarithm is squared dist.
-        """
+        """Test that squared norm of logarithm is squared dist."""
         tangent_vec = self.l2_metric_s2.log(
-            landmarks=self.landmarks_b, base_landmarks=self.landmarks_a)
+            point=self.landmarks_b, base_point=self.landmarks_a)
         log_ab = tangent_vec
         result = self.l2_metric_s2.squared_norm(
             vector=log_ab, base_point=self.landmarks_a)
@@ -78,25 +70,19 @@ class TestLandmarksMethods(geomstats.tests.TestCase):
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_l2_metric_log_and_exp(self):
-        """
-        Test that exp and log are inverse maps.
-        """
+        """Test that exp and log are inverse maps."""
         tangent_vec = self.l2_metric_s2.log(
-            landmarks=self.landmarks_b, base_landmarks=self.landmarks_a)
+            point=self.landmarks_b, base_point=self.landmarks_a)
         result = self.l2_metric_s2.exp(
-            tangent_vec=tangent_vec,
-            base_landmarks=self.landmarks_a)
+            tangent_vec=tangent_vec, base_point=self.landmarks_a)
         expected = self.landmarks_b
 
         self.assertAllClose(result, expected, atol=self.atol)
 
-    @geomstats.tests.np_only
+    @geomstats.tests.np_and_tf_only
     def test_l2_metric_inner_product_vectorization(self):
-        """
-        Test the vectorization inner_product.
-        """
+        """Test the vectorization inner_product."""
         n_samples = self.n_landmark_sets
         landmarks_ab = self.l2_metric_s2.geodesic(
             self.landmarks_a, self.landmarks_b)
@@ -106,18 +92,16 @@ class TestLandmarksMethods(geomstats.tests.TestCase):
         landmarks_bc = landmarks_bc(self.times)
 
         tangent_vecs = self.l2_metric_s2.log(
-            landmarks=landmarks_bc, base_landmarks=landmarks_ab)
+            point=landmarks_bc, base_point=landmarks_ab)
 
         result = self.l2_metric_s2.inner_product(
             tangent_vecs, tangent_vecs, landmarks_ab)
 
         self.assertAllClose(gs.shape(result), (n_samples,))
 
-    @geomstats.tests.np_only
+    @geomstats.tests.np_and_tf_only
     def test_l2_metric_dist_vectorization(self):
-        """
-        Test the vectorization of dist.
-        """
+        """Test the vectorization of dist."""
         n_samples = self.n_landmark_sets
         landmarks_ab = self.l2_metric_s2.geodesic(
             self.landmarks_a, self.landmarks_b)
@@ -132,9 +116,7 @@ class TestLandmarksMethods(geomstats.tests.TestCase):
 
     @geomstats.tests.np_and_tf_only
     def test_l2_metric_exp_vectorization(self):
-        """
-        Test the vectorization of exp.
-        """
+        """Test the vectorization of exp."""
         landmarks_ab = self.l2_metric_s2.geodesic(
             self.landmarks_a, self.landmarks_b)
         landmarks_bc = self.l2_metric_s2.geodesic(
@@ -143,17 +125,15 @@ class TestLandmarksMethods(geomstats.tests.TestCase):
         landmarks_bc = landmarks_bc(self.times)
 
         tangent_vecs = self.l2_metric_s2.log(
-            landmarks=landmarks_bc, base_landmarks=landmarks_ab)
+            point=landmarks_bc, base_point=landmarks_ab)
 
         result = self.l2_metric_s2.exp(
-            tangent_vec=tangent_vecs, base_landmarks=landmarks_ab)
+            tangent_vec=tangent_vecs, base_point=landmarks_ab)
         self.assertAllClose(gs.shape(result), gs.shape(landmarks_ab))
 
-    @geomstats.tests.np_only
+    @geomstats.tests.np_and_tf_only
     def test_l2_metric_log_vectorization(self):
-        """
-        Test the vectorization of log.
-        """
+        """Test the vectorization of log."""
         landmarks_ab = self.l2_metric_s2.geodesic(
             self.landmarks_a, self.landmarks_b)
         landmarks_bc = self.l2_metric_s2.geodesic(
@@ -162,33 +142,25 @@ class TestLandmarksMethods(geomstats.tests.TestCase):
         landmarks_bc = landmarks_bc(self.times)
 
         tangent_vecs = self.l2_metric_s2.log(
-            landmarks=landmarks_bc, base_landmarks=landmarks_ab)
+            point=landmarks_bc, base_point=landmarks_ab)
 
         result = tangent_vecs
         self.assertAllClose(gs.shape(result), gs.shape(landmarks_ab))
 
-    @geomstats.tests.np_only
+    @geomstats.tests.np_and_tf_only
     def test_l2_metric_geodesic(self):
-        """
-        Test the geodesic method of L2Metric.
-        """
+        """Test the geodesic method of L2Metric."""
         landmarks_ab = self.l2_metric_s2.geodesic(
             self.landmarks_a, self.landmarks_b)
-        landmarks_bc = self.l2_metric_s2.geodesic(
-            self.landmarks_b, self.landmarks_c)
         landmarks_ab = landmarks_ab(self.times)
-        landmarks_bc = landmarks_bc(self.times)
 
         result = landmarks_ab
-        expected = gs.zeros(landmarks_ab.shape)
+        expected = []
         for k in range(self.n_sampling_points):
             geod = self.l2_metric_s2.ambient_metric.geodesic(
                 initial_point=self.landmarks_a[k, :],
                 end_point=self.landmarks_b[k, :])
-            expected[:, k, :] = geod(self.times)
+            expected.append(geod(self.times))
+        expected = gs.stack(expected, axis=1)
 
         self.assertAllClose(result, expected)
-
-        geod = self.l2_metric_s2.geodesic(
-            initial_landmarks=landmarks_ab,
-            end_landmarks=landmarks_bc)
