@@ -1,6 +1,5 @@
-"""
-Unit tests for Minkowski space.
-"""
+"""Unit tests for Minkowski space."""
+
 import math
 
 import numpy as np
@@ -10,7 +9,7 @@ import geomstats.tests
 from geomstats.geometry.minkowski import Minkowski
 
 
-class TestMinkowskiMethods(geomstats.tests.TestCase):
+class TestMinkowski(geomstats.tests.TestCase):
     def setUp(self):
         gs.random.seed(1234)
 
