@@ -7,7 +7,7 @@ import geomstats.tests
 import geomstats.vectorization
 
 
-class TestVectorizationMethods(geomstats.tests.TestCase):
+class TestVectorization(geomstats.tests.TestCase):
     def setUp(self):
 
         class Obj:
@@ -15,21 +15,21 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
                 self.default_point_type = 'vector'
 
         @geomstats.vectorization.decorator(['vector', 'vector'])
-        def foo(tangent_vec_a, tangent_vec_b):
+        def func(tangent_vec_a, tangent_vec_b):
             result = gs.einsum(
                 '...i,...i->...i', tangent_vec_a, tangent_vec_b)
             result = helper.to_vector(result)
             return result
 
         @geomstats.vectorization.decorator(['vector', 'vector'])
-        def foo_scalar_output(tangent_vec_a, tangent_vec_b):
+        def func_scalar_output(tangent_vec_a, tangent_vec_b):
             result = gs.einsum(
                 '...i,...i->...', tangent_vec_a, tangent_vec_b)
             result = helper.to_scalar(result)
             return result
 
         @geomstats.vectorization.decorator(['vector', 'vector', 'scalar'])
-        def foo_scalar_input_output(tangent_vec_a, tangent_vec_b, in_scalar):
+        def func_scalar_input_output(tangent_vec_a, tangent_vec_b, in_scalar):
             aux = gs.einsum(
                 'ni,ni->n', tangent_vec_a, tangent_vec_b)
             result = gs.einsum('n,nk->n', aux, in_scalar)
@@ -37,7 +37,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
             return result
 
         @geomstats.vectorization.decorator(['vector', 'vector', 'scalar'])
-        def foo_optional_input(tangent_vec_a, tangent_vec_b, in_scalar=None):
+        def func_optional_input(tangent_vec_a, tangent_vec_b, in_scalar=None):
             if in_scalar is None:
                 in_scalar = gs.array([[1]])
             aux = gs.einsum(
@@ -48,7 +48,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
 
         @geomstats.vectorization.decorator(
             ['else', 'vector', 'else', 'vector', 'scalar'])
-        def foo_else(else_a, tangent_vec_a, else_b, tangent_vec_b):
+        def func_else(else_a, tangent_vec_a, else_b, tangent_vec_b):
             result = (else_a + else_b) * gs.einsum(
                 'ni,ni->n', tangent_vec_a, tangent_vec_b)
             result = helper.to_scalar(result)
@@ -109,11 +109,11 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
             result = gs.array([[5.]] * n_samples)
             return result
 
-        self.foo = foo
-        self.foo_scalar_output = foo_scalar_output
-        self.foo_scalar_input_output = foo_scalar_input_output
-        self.foo_optional_input = foo_optional_input
-        self.foo_else = foo_else
+        self.func = func
+        self.func_scalar_output = func_scalar_output
+        self.func_scalar_input_output = func_scalar_input_output
+        self.func_optional_input = func_optional_input
+        self.func_else = func_else
         self.is_scalar_vectorized = is_scalar_vectorized
         self.is_vector_vectorized = is_vector_vectorized
         self.is_matrix_vectorized = is_matrix_vectorized
@@ -129,7 +129,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
     def test_decorator_with_squeeze_dim0(self):
         vec_a = gs.array([1, 2, 3])
         vec_b = gs.array([0, 1, 0])
-        result = self.foo(vec_a, vec_b)
+        result = self.func(vec_a, vec_b)
         expected = gs.array([0, 2, 0])
 
         self.assertAllClose(result.shape, expected.shape)
@@ -138,7 +138,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
     def test_decorator_with_squeeze_dim0_with_kwargs(self):
         vec_a = gs.array([1, 2, 3])
         vec_b = gs.array([0, 1, 0])
-        result = self.foo(tangent_vec_a=vec_a, tangent_vec_b=vec_b)
+        result = self.func(tangent_vec_a=vec_a, tangent_vec_b=vec_b)
         expected = gs.array([0, 2, 0])
 
         self.assertAllClose(result.shape, expected.shape)
@@ -147,7 +147,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
     def test_decorator_without_squeeze_dim0(self):
         vec_a = gs.array([[1, 2, 3]])
         vec_b = gs.array([0, 1, 0])
-        result = self.foo(vec_a, vec_b)
+        result = self.func(vec_a, vec_b)
         expected = gs.array([[0, 2, 0]])
 
         self.assertAllClose(result.shape, expected.shape)
@@ -156,7 +156,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
     def test_decorator_without_squeeze_dim0_with_kwargs(self):
         vec_a = gs.array([[1, 2, 3]])
         vec_b = gs.array([0, 1, 0])
-        result = self.foo(tangent_vec_a=vec_a, tangent_vec_b=vec_b)
+        result = self.func(tangent_vec_a=vec_a, tangent_vec_b=vec_b)
         expected = gs.array([[0, 2, 0]])
 
         self.assertAllClose(result.shape, expected.shape)
@@ -165,7 +165,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
     def test_decorator_vectorization(self):
         vec_a = gs.array([[1, 2, 3], [1, 2, 3]])
         vec_b = gs.array([0, 1, 0])
-        result = self.foo(vec_a, vec_b)
+        result = self.func(vec_a, vec_b)
         expected = gs.array([[0, 2, 0], [0, 2, 0]])
 
         self.assertAllClose(result.shape, expected.shape)
@@ -174,7 +174,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
     def test_decorator_vectorization_with_kwargs(self):
         vec_a = gs.array([[1, 2, 3], [1, 2, 3]])
         vec_b = gs.array([0, 1, 0])
-        result = self.foo(tangent_vec_a=vec_a, tangent_vec_b=vec_b)
+        result = self.func(tangent_vec_a=vec_a, tangent_vec_b=vec_b)
         expected = gs.array([[0, 2, 0], [0, 2, 0]])
 
         self.assertAllClose(result.shape, expected.shape)
@@ -183,7 +183,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
     def test_decorator_scalar_with_squeeze_dim1(self):
         vec_a = gs.array([1, 2, 3])
         vec_b = gs.array([0, 1, 0])
-        result = self.foo_scalar_output(vec_a, vec_b)
+        result = self.func_scalar_output(vec_a, vec_b)
         expected = 2
 
         self.assertAllClose(result, expected)
@@ -191,7 +191,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
     def test_decorator_scalar_with_squeeze_dim1_with_kwargs(self):
         vec_a = gs.array([1, 2, 3])
         vec_b = gs.array([0, 1, 0])
-        result = self.foo_scalar_output(
+        result = self.func_scalar_output(
             tangent_vec_a=vec_a, tangent_vec_b=vec_b)
         expected = 2
 
@@ -201,7 +201,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
         vec_a = gs.array([1, 2, 3])
         vec_b = gs.array([0, 1, 0])
         scalar = 4
-        result = self.foo_scalar_input_output(vec_a, vec_b, scalar)
+        result = self.func_scalar_input_output(vec_a, vec_b, scalar)
         expected = 8
 
         self.assertAllClose(result, expected)
@@ -210,7 +210,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
         vec_a = gs.array([1, 2, 3])
         vec_b = gs.array([0, 1, 0])
         scalar = 4
-        result = self.foo_scalar_input_output(
+        result = self.func_scalar_input_output(
             tangent_vec_a=vec_a, tangent_vec_b=vec_b, in_scalar=scalar)
         expected = 8
 
@@ -219,7 +219,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
     def test_decorator_scalar_output_vectorization(self):
         vec_a = gs.array([[1, 2, 3], [1, 2, 3]])
         vec_b = gs.array([0, 1, 0])
-        result = self.foo_scalar_output(vec_a, vec_b)
+        result = self.func_scalar_output(vec_a, vec_b)
         expected = gs.array([2, 2])
 
         self.assertAllClose(result.shape, expected.shape)
@@ -228,7 +228,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
     def test_decorator_scalar_output_vectorization_with_kwargs(self):
         vec_a = gs.array([[1, 2, 3], [1, 2, 3]])
         vec_b = gs.array([0, 1, 0])
-        result = self.foo_scalar_output(
+        result = self.func_scalar_output(
             tangent_vec_a=vec_a, tangent_vec_b=vec_b)
         expected = gs.array([2, 2])
 
@@ -238,7 +238,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
     def test_decorator_optional_input(self):
         vec_a = gs.array([1, 2, 3])
         vec_b = gs.array([0, 1, 0])
-        result = self.foo_optional_input(vec_a, vec_b)
+        result = self.func_optional_input(vec_a, vec_b)
         expected = 2
 
         self.assertAllClose(result, expected)
@@ -246,7 +246,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
     def test_decorator_optional_input_with_kwargs(self):
         vec_a = gs.array([1, 2, 3])
         vec_b = gs.array([0, 1, 0])
-        result = self.foo_optional_input(
+        result = self.func_optional_input(
             tangent_vec_a=vec_a, tangent_vec_b=vec_b, in_scalar=3)
         expected = 6
 
@@ -255,7 +255,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
     def test_decorator_optional_input_with_optional_kwargs(self):
         vec_a = gs.array([1, 2, 3])
         vec_b = gs.array([0, 1, 0])
-        result = self.foo_optional_input(
+        result = self.func_optional_input(
             tangent_vec_a=vec_a, tangent_vec_b=vec_b)
         expected = 2
 
@@ -266,7 +266,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
         vec_b = gs.array([0, 1, 0])
         else_a = 1
         else_b = 1
-        result = self.foo_else(else_a, vec_a, else_b, vec_b)
+        result = self.func_else(else_a, vec_a, else_b, vec_b)
         expected = 4
 
         self.assertAllClose(result, expected)
@@ -276,7 +276,7 @@ class TestVectorizationMethods(geomstats.tests.TestCase):
         vec_b = gs.array([0, 1, 0])
         else_a = 1
         else_b = 1
-        result = self.foo_else(
+        result = self.func_else(
             else_a=else_a, tangent_vec_a=vec_a,
             else_b=else_b, tangent_vec_b=vec_b)
         expected = 4
